/*  Spruce
 *  Copyright (C) 1999-2000 Jeffrey Stedfast
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "composer.h"
#include "gti.h"

/* icons */
#include "xpm/send.xpm"
#include "xpm/csave.xpm"
#include "xpm/print.xpm"
#include "xpm/attach.xpm"
#if defined(ENABLE_GPG) || defined (ENABLE_PGP)
#	include "xpm/encrypt.xpm"
#endif

/* globals for Composer */
static gchar sprucesig[] = ".spruce/sprucesig";
static gchar xmailer[] = "Spruce " VERSION " for X11";
static gint SaveAsOpen = 0;
static gchar *SaveFile;                  /* file name for email being saved */
static GtkWidget *frmComposeSaveAs;

/* external variables */
extern gboolean AboutOpen;               /* is frmAbout opened?? */
extern GtkWidget *frmAbout;
extern gboolean AddrBookOpen;            /* is frmAddrBook opened?? */
extern GtkWidget *frmAddrBook;
extern gchar *AddrSelected;              /* our returned addr from frmAddrBook */
extern GtkWidget *frmComposer;
extern gboolean ComposerOpen;            /* is frmComposer opened?? */
extern gboolean PassphraseOpen;
extern gint use_rand_sig;
extern gint auto_attach_sig;
extern gint save_outgoing;
extern gchar *signaturefile;
extern Server SmtpServer;
extern gchar *SenderAddr;
extern gchar *ReplyTo;
extern gint texticons;
extern gint word_wrap_len;

extern gint pgp_sign;
extern gint use_gnupg;
extern gint store_passphrase;
extern gint encrypt_to_self;
extern gchar *pgp_identity;
extern gchar *pgp_passphrase;

extern gchar *mesg_body_font;

extern GList *mime_send_parts;

GtkWidget* create_frmComposer (gchar *sender, gchar *recipient, gchar *cc, gchar *bcc, gchar *subject, gchar *body)
{
  GtkWidget *frmComposer;
  GtkWidget *vbox1;
  GtkWidget *menubar;
  GtkWidget *mnuFile;
  GtkWidget *mnuFile_menu;
  GtkAccelGroup *mnuFile_menu_accels;
  GtkWidget *mnuComposerFileSend;
  GtkWidget *mnuComposerFileSave;
  GtkWidget *mnuComposerFileSave_As;
  GtkWidget *mnuComposerFilePrint;
  GtkWidget *separator1;
  GtkWidget *mnuComposerFileClose;
  GtkWidget *mnuEdit;
  GtkWidget *mnuEdit_menu;
  GtkAccelGroup *mnuEdit_menu_accels;
  GtkWidget *mnuComposerEditCut;
  GtkWidget *mnuComposerEditCopy;
  GtkWidget *mnuComposerEditPaste;
  GtkWidget *separator2;
  GtkWidget *mnuComposerEditInsert;
  GtkWidget *mnuComposerEditInsert_menu;
  GtkAccelGroup *mnuComposerEditInsert_menu_accels;
  GtkWidget *mnuComposerInsertFile;
  GtkWidget *mnuComposerInsertAttachment;
  GtkWidget *separator3;
  GtkWidget *mnuComposerInsertSignature;
  GtkWidget *mnuComposerInsertSpruce_Sig;
  GtkWidget *mnuComposerHelp;
  GtkWidget *mnuComposerHelp_menu;
  GtkAccelGroup *mnuComposerHelp_menu_accels;
  GtkWidget *mnuComposerHelpAbout;
  GtkWidget *handlebox;
  GtkWidget *frame1;
  GtkWidget *toolbar;
  GtkWidget *cmdComposerSend;
  GtkWidget *cmdComposerSave;
  GtkWidget *cmdComposerPrint;
  GtkWidget *cmdComposerAttach;
#if defined(ENABLE_GPG) || defined (ENABLE_PGP)
  GtkWidget *cmdComposerEncrypt;
#endif
  GtkWidget *vbox2;
  GtkWidget *notebook;
  GtkWidget *table1;
  GtkWidget *hbox3;
  GtkWidget *txtTo;
  GtkWidget *cmdToAddrBook;
  GtkWidget *hbox4;
  GtkWidget *txtCC;
  GtkWidget *cmdCCAddrBook;
  GtkWidget *hbox5;
  GtkWidget *txtBCC;
  GtkWidget *cmdBCCAddrBook;
  GtkWidget *cmboFrom;
  GtkWidget *txtFrom;
  GtkWidget *lblTo;
  GtkWidget *lblFrom;
  GtkWidget *lblCC;
  GtkWidget *lblBCC;
  GtkWidget *lblAddress;
  GtkWidget *scrollAttachments;
  GtkWidget *clistAttachments;
  GtkWidget *lblSize;
  GtkWidget *lblFileName;
  GtkWidget *lblAttachments;
  GtkWidget *hbox6;
  GtkWidget *vbox4;
  GtkWidget *lblOptions;
  GtkWidget *hbox2;
  GtkWidget *hbox10;
  GtkWidget *lblSubject;
  GtkWidget *txtSubject;
  GtkWidget *hbox9;
  GtkWidget *lblPriority;
  GtkWidget *optPriority;
  GtkWidget *optPriority_menu;
  GtkWidget *glade_menuitem;
  GtkWidget *frame2;
  GtkWidget *scrollBody;
  GtkWidget *txtBody;
  GtkWidget *hbox1;
  GtkWidget *progressComposerSend;
  GtkWidget *statusbar;

  GtkWidget *iconw;
  GdkBitmap *mask;
  GdkPixmap *icon;
  GdkFont *fixed_font = NULL;

  frmComposer = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (frmComposer, "frmComposer");
  gtk_object_set_data (GTK_OBJECT (frmComposer), "frmComposer", frmComposer);
  gtk_widget_set_usize (frmComposer, 512, 550);
  gtk_signal_connect_object (GTK_OBJECT (frmComposer), "destroy",
                             GTK_SIGNAL_FUNC (on_frmComposer_destroy),
                             GTK_OBJECT (frmComposer));
  gtk_signal_connect_object (GTK_OBJECT (frmComposer), "show",
                             GTK_SIGNAL_FUNC (on_frmComposer_show),
                             GTK_OBJECT (frmComposer));
  gtk_window_set_title (GTK_WINDOW (frmComposer), _("Spruce Composer"));
  gtk_window_set_policy (GTK_WINDOW (frmComposer), TRUE, TRUE, FALSE);
  gtk_widget_realize(frmComposer);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbox1, "vbox1");
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (frmComposer), vbox1);

  menubar = gtk_menu_bar_new ();
  gtk_widget_set_name (menubar, "menubar");
  gtk_widget_ref (menubar);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "menubar", menubar,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menubar);
  gtk_box_pack_start (GTK_BOX (vbox1), menubar, FALSE, FALSE, 0);

  mnuFile = gtk_menu_item_new_with_label (_("File"));
  gtk_widget_set_name (mnuFile, "mnuFile");
  gtk_widget_ref (mnuFile);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuFile", mnuFile,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (mnuFile);
  gtk_container_add (GTK_CONTAINER (menubar), mnuFile);

  mnuFile_menu = gtk_menu_new ();
  gtk_widget_set_name (mnuFile_menu, "mnuFile_menu");
  gtk_widget_ref (mnuFile_menu);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuFile_menu", mnuFile_menu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (mnuFile), mnuFile_menu);
  mnuFile_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (mnuFile_menu));

  mnuComposerFileSend = gtk_menu_item_new_with_label (_("Send"));
  gtk_widget_set_name (mnuComposerFileSend, "mnuComposerFileSend");
  gtk_widget_ref (mnuComposerFileSend);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuComposerFileSend", mnuComposerFileSend,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (mnuComposerFileSend), "activate",
                      GTK_SIGNAL_FUNC (on_mnuComposerFileSend_activate),
                      GTK_OBJECT (frmComposer));
  gtk_widget_show (mnuComposerFileSend);
  gtk_container_add (GTK_CONTAINER (mnuFile_menu), mnuComposerFileSend);

  mnuComposerFileSave = gtk_menu_item_new_with_label (_("Save"));
  gtk_widget_set_name (mnuComposerFileSave, "mnuComposerFileSave");
  gtk_widget_ref (mnuComposerFileSave);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuComposerFileSave", mnuComposerFileSave,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (mnuComposerFileSave), "activate",
                      GTK_SIGNAL_FUNC (on_mnuComposerFileSave_activate),
                      GTK_OBJECT (frmComposer));
  gtk_widget_show (mnuComposerFileSave);
  gtk_container_add (GTK_CONTAINER (mnuFile_menu), mnuComposerFileSave);

  mnuComposerFileSave_As = gtk_menu_item_new_with_label (_("Save As..."));
  gtk_widget_set_name (mnuComposerFileSave_As, "mnuComposerFileSave_As");
  gtk_widget_ref (mnuComposerFileSave_As);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuComposerFileSave_As", mnuComposerFileSave_As,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (mnuComposerFileSave_As), "activate",
                      GTK_SIGNAL_FUNC (on_mnuComposerFileSave_As_activate),
                      GTK_OBJECT (frmComposer));
  gtk_widget_show (mnuComposerFileSave_As);
  gtk_container_add (GTK_CONTAINER (mnuFile_menu), mnuComposerFileSave_As);

  mnuComposerFilePrint = gtk_menu_item_new_with_label (_("Print"));
  gtk_widget_set_name (mnuComposerFilePrint, "mnuComposerFilePrint");
  gtk_widget_ref (mnuComposerFilePrint);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuComposerFilePrint", mnuComposerFilePrint,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (mnuComposerFilePrint), "activate",
                      GTK_SIGNAL_FUNC (on_mnuComposerFilePrint_activate),
                      GTK_OBJECT (frmComposer));
  gtk_widget_show (mnuComposerFilePrint);
  gtk_container_add (GTK_CONTAINER (mnuFile_menu), mnuComposerFilePrint);

  separator1 = gtk_menu_item_new ();
  gtk_widget_set_name (separator1, "separator1");
  gtk_widget_ref (separator1);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "separator1", separator1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (separator1);
  gtk_container_add (GTK_CONTAINER (mnuFile_menu), separator1);
  gtk_widget_set_sensitive (separator1, FALSE);

  mnuComposerFileClose = gtk_menu_item_new_with_label (_("Close"));
  gtk_widget_set_name (mnuComposerFileClose, "mnuComposerFileClose");
  gtk_widget_ref (mnuComposerFileClose);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuComposerFileClose", mnuComposerFileClose,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (mnuComposerFileClose), "activate",
                      GTK_SIGNAL_FUNC (on_mnuComposerFileClose_activate),
                      GTK_OBJECT (frmComposer));
  gtk_widget_show (mnuComposerFileClose);
  gtk_container_add (GTK_CONTAINER (mnuFile_menu), mnuComposerFileClose);

  mnuEdit = gtk_menu_item_new_with_label (_("Edit"));
  gtk_widget_set_name (mnuEdit, "mnuEdit");
  gtk_widget_ref (mnuEdit);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuEdit", mnuEdit,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (mnuEdit);
  gtk_container_add (GTK_CONTAINER (menubar), mnuEdit);

  mnuEdit_menu = gtk_menu_new ();
  gtk_widget_set_name (mnuEdit_menu, "mnuEdit_menu");
  gtk_widget_ref (mnuEdit_menu);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuEdit_menu", mnuEdit_menu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (mnuEdit), mnuEdit_menu);
  mnuEdit_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (mnuEdit_menu));

  mnuComposerEditCut = gtk_menu_item_new_with_label (_("Cut"));
  gtk_widget_set_name (mnuComposerEditCut, "mnuComposerEditCut");
  gtk_widget_ref (mnuComposerEditCut);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuComposerEditCut", mnuComposerEditCut,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (mnuComposerEditCut), "activate",
                      GTK_SIGNAL_FUNC (on_mnuComposerEditCut_activate),
                      GTK_OBJECT (frmComposer));
  gtk_widget_show (mnuComposerEditCut);
  gtk_container_add (GTK_CONTAINER (mnuEdit_menu), mnuComposerEditCut);

  mnuComposerEditCopy = gtk_menu_item_new_with_label (_("Copy"));
  gtk_widget_set_name (mnuComposerEditCopy, "mnuComposerEditCopy");
  gtk_widget_ref (mnuComposerEditCopy);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuComposerEditCopy", mnuComposerEditCopy,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (mnuComposerEditCopy), "activate",
                      GTK_SIGNAL_FUNC (on_mnuComposerEditCopy_activate),
                      GTK_OBJECT (frmComposer));
  gtk_widget_show (mnuComposerEditCopy);
  gtk_container_add (GTK_CONTAINER (mnuEdit_menu), mnuComposerEditCopy);

  mnuComposerEditPaste = gtk_menu_item_new_with_label (_("Paste"));
  gtk_widget_set_name (mnuComposerEditPaste, "mnuComposerEditPaste");
  gtk_widget_ref (mnuComposerEditPaste);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuComposerEditPaste", mnuComposerEditPaste,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (mnuComposerEditPaste), "activate",
                      GTK_SIGNAL_FUNC (on_mnuComposerEditPaste_activate),
                      GTK_OBJECT (frmComposer));
  gtk_widget_show (mnuComposerEditPaste);
  gtk_container_add (GTK_CONTAINER (mnuEdit_menu), mnuComposerEditPaste);

  separator2 = gtk_menu_item_new ();
  gtk_widget_set_name (separator2, "separator2");
  gtk_widget_ref (separator2);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "separator2", separator2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (separator2);
  gtk_container_add (GTK_CONTAINER (mnuEdit_menu), separator2);
  gtk_widget_set_sensitive (separator2, FALSE);

  mnuComposerEditInsert = gtk_menu_item_new_with_label (_("Insert"));
  gtk_widget_set_name (mnuComposerEditInsert, "mnuComposerEditInsert");
  gtk_widget_ref (mnuComposerEditInsert);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuComposerEditInsert", mnuComposerEditInsert,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (mnuComposerEditInsert);
  gtk_container_add (GTK_CONTAINER (mnuEdit_menu), mnuComposerEditInsert);

  mnuComposerEditInsert_menu = gtk_menu_new ();
  gtk_widget_set_name (mnuComposerEditInsert_menu, "mnuComposerEditInsert_menu");
  gtk_widget_ref (mnuComposerEditInsert_menu);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuComposerEditInsert_menu", mnuComposerEditInsert_menu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (mnuComposerEditInsert), mnuComposerEditInsert_menu);
  mnuComposerEditInsert_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (mnuComposerEditInsert_menu));

  mnuComposerInsertFile = gtk_menu_item_new_with_label (_("Insert File..."));
  gtk_widget_set_name (mnuComposerInsertFile, "mnuComposerInsertFile");
  gtk_widget_ref (mnuComposerInsertFile);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuComposerInsertFile", mnuComposerInsertFile,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (mnuComposerInsertFile), "activate",
                      GTK_SIGNAL_FUNC (on_mnuComposerInsertFile_activate),
                      GTK_OBJECT(frmComposer));
  gtk_widget_show (mnuComposerInsertFile);
  gtk_container_add (GTK_CONTAINER (mnuComposerEditInsert_menu), mnuComposerInsertFile);

  mnuComposerInsertAttachment = gtk_menu_item_new_with_label (_("Attach File..."));
  gtk_widget_set_name (mnuComposerInsertAttachment, "mnuComposerInsertAttachment");
  gtk_widget_ref (mnuComposerInsertAttachment);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuComposerInsertAttachment", mnuComposerInsertAttachment,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (mnuComposerInsertAttachment), "activate",
                      GTK_SIGNAL_FUNC (on_mnuComposerInsertAttachment_activate),
                      GTK_OBJECT(frmComposer));
  gtk_widget_show (mnuComposerInsertAttachment);
  gtk_container_add (GTK_CONTAINER (mnuComposerEditInsert_menu), mnuComposerInsertAttachment);

  separator3 = gtk_menu_item_new ();
  gtk_widget_set_name (separator3, "separator3");
  gtk_widget_ref (separator3);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "separator3", separator3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (separator3);
  gtk_container_add (GTK_CONTAINER (mnuComposerEditInsert_menu), separator3);
  gtk_widget_set_sensitive (separator3, FALSE);

  mnuComposerInsertSignature = gtk_menu_item_new_with_label (_("Signature"));
  gtk_widget_set_name (mnuComposerInsertSignature, "mnuComposerInsertSignature");
  gtk_widget_ref (mnuComposerInsertSignature);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuComposerInsertSignature", mnuComposerInsertSignature,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (mnuComposerInsertSignature), "activate",
                      GTK_SIGNAL_FUNC (on_mnuComposerInsertSignature_activate),
                      GTK_OBJECT(frmComposer));
  gtk_widget_show (mnuComposerInsertSignature);
  gtk_container_add (GTK_CONTAINER (mnuComposerEditInsert_menu), mnuComposerInsertSignature);

  mnuComposerInsertSpruce_Sig = gtk_menu_item_new_with_label (_("Spruce Sig"));
  gtk_widget_set_name (mnuComposerInsertSpruce_Sig, "mnuComposerInsertSpruce_Sig");
  gtk_widget_ref (mnuComposerInsertSpruce_Sig);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuComposerInsertSpruce_Sig", mnuComposerInsertSpruce_Sig,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (mnuComposerInsertSpruce_Sig), "activate",
                      GTK_SIGNAL_FUNC (on_mnuComposerInsertSpruce_Sig_activate),
                      GTK_OBJECT(frmComposer));
  gtk_widget_show (mnuComposerInsertSpruce_Sig);
  gtk_container_add (GTK_CONTAINER (mnuComposerEditInsert_menu), mnuComposerInsertSpruce_Sig);

  mnuComposerHelp = gtk_menu_item_new_with_label (_("Help"));
  gtk_widget_set_name (mnuComposerHelp, "mnuComposerHelp");
  gtk_widget_ref (mnuComposerHelp);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuComposerHelp", mnuComposerHelp,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (mnuComposerHelp);
  gtk_container_add (GTK_CONTAINER (menubar), mnuComposerHelp);

  mnuComposerHelp_menu = gtk_menu_new ();
  gtk_widget_set_name (mnuComposerHelp_menu, "mnuComposerHelp_menu");
  gtk_widget_ref (mnuComposerHelp_menu);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuComposerHelp_menu", mnuComposerHelp_menu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (mnuComposerHelp), mnuComposerHelp_menu);
  mnuComposerHelp_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (mnuComposerHelp_menu));

  mnuComposerHelpAbout = gtk_menu_item_new_with_label (_("About"));
  gtk_widget_set_name (mnuComposerHelpAbout, "mnuComposerHelpAbout");
  gtk_widget_ref (mnuComposerHelpAbout);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "mnuComposerHelpAbout", mnuComposerHelpAbout,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (mnuComposerHelpAbout), "activate",
                      GTK_SIGNAL_FUNC (on_mnuComposerHelpAbout_activate),
                      NULL);
  gtk_widget_show (mnuComposerHelpAbout);
  gtk_container_add (GTK_CONTAINER (mnuComposerHelp_menu), mnuComposerHelpAbout);

  handlebox = gtk_handle_box_new ();
  gtk_widget_set_name (handlebox, "handlebox");
  gtk_widget_ref (handlebox);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "handlebox", handlebox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (handlebox);
  gtk_box_pack_start (GTK_BOX (vbox1), handlebox, FALSE, TRUE, 0);
  gtk_handle_box_set_snap_edge (GTK_HANDLE_BOX (handlebox), GTK_POS_LEFT);

  frame1 = gtk_frame_new (NULL);
  gtk_widget_set_name (frame1, "frame1");
  gtk_widget_ref (frame1);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "frame1", frame1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame1);
  gtk_container_add (GTK_CONTAINER (handlebox), frame1);
  gtk_frame_set_shadow_type (GTK_FRAME (frame1), GTK_SHADOW_OUT);

  switch (texticons)
  {
     case (TB_TEXT):
        toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_TEXT);
        break;
     case (TB_ICONS):
        toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
        break;
     case (TB_BOTH):
        toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
        break;
     default:
        toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
        break;
  }
  gtk_widget_set_name (toolbar, "toolbar");
  gtk_widget_ref (toolbar);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "toolbar", toolbar,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_add (GTK_CONTAINER (frame1), toolbar);
  gtk_toolbar_set_space_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_SPACE_LINE);
  gtk_toolbar_set_button_relief (GTK_TOOLBAR (toolbar), GTK_RELIEF_NONE);
  gtk_widget_show (toolbar);

  icon = gdk_pixmap_create_from_xpm_d (frmComposer->window, &mask,
               &frmComposer->style->white, send_xpm);
  iconw = gtk_pixmap_new(icon, mask);
  cmdComposerSend = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Send"),
                                _("Send Message"), "Private",
                                iconw, NULL, NULL);
  gtk_signal_connect_object (GTK_OBJECT (cmdComposerSend), "clicked",
                             GTK_SIGNAL_FUNC (on_cmdComposerSend_clicked),
                             GTK_OBJECT (frmComposer));
  gtk_widget_set_name (cmdComposerSend, "cmdComposerSend");
  gtk_widget_ref (cmdComposerSend);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "cmdComposerSend", cmdComposerSend,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cmdComposerSend);

  icon = gdk_pixmap_create_from_xpm_d (frmComposer->window, &mask,
               &frmComposer->style->white, csave_xpm);
  iconw = gtk_pixmap_new(icon, mask);
  cmdComposerSave = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Save"),
                                _("Save Message"), "Private",
                                iconw, NULL, NULL);
  gtk_widget_set_name (cmdComposerSave, "cmdComposerSave");
  gtk_widget_ref (cmdComposerSave);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "cmdComposerSave", cmdComposerSave,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (cmdComposerSave), "clicked",
                             GTK_SIGNAL_FUNC (on_cmdComposerSave_clicked),
                             GTK_OBJECT (frmComposer));
  gtk_widget_show (cmdComposerSave);

  icon = gdk_pixmap_create_from_xpm_d (frmComposer->window, &mask,
               &frmComposer->style->white, print_xpm);
  iconw = gtk_pixmap_new(icon, mask);
  cmdComposerPrint = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Print"),
                                _("Print Message"), "Private",
                                iconw, NULL, NULL);
  gtk_signal_connect_object (GTK_OBJECT (cmdComposerPrint), "clicked",
                             GTK_SIGNAL_FUNC (on_cmdComposerPrint_clicked),
                             GTK_OBJECT (frmComposer));
  gtk_widget_set_name (cmdComposerPrint, "cmdComposerPrint");
  gtk_widget_ref (cmdComposerPrint);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "cmdComposerPrint", cmdComposerPrint,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cmdComposerPrint);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));

  icon = gdk_pixmap_create_from_xpm_d (frmComposer->window, &mask,
               &frmComposer->style->white, attach_xpm);
  iconw = gtk_pixmap_new(icon, mask);
  cmdComposerAttach = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Attach"),
                                _("Attach File"), "Private",
                                iconw, NULL, NULL);
  gtk_signal_connect_object (GTK_OBJECT (cmdComposerAttach), "clicked",
                             GTK_SIGNAL_FUNC (on_cmdComposerAttach_clicked),
                             GTK_OBJECT (frmComposer));
  gtk_widget_set_name (cmdComposerAttach, "cmdComposerAttach");
  gtk_widget_ref (cmdComposerAttach);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "cmdComposerAttach", cmdComposerAttach,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cmdComposerAttach);

#if defined(ENABLE_GPG) || defined (ENABLE_PGP)
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
  
  icon = gdk_pixmap_create_from_xpm_d (frmComposer->window, &mask,
               &frmComposer->style->white, encrypt_xpm);
  iconw = gtk_pixmap_new(icon, mask);
  cmdComposerEncrypt = gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Encrypt"),
                                _("PGP Encrypt"), "Private",
                                iconw, NULL, NULL);
  gtk_signal_connect_object (GTK_OBJECT (cmdComposerEncrypt), "clicked",
                             GTK_SIGNAL_FUNC (on_cmdComposerEncrypt_clicked),
                             GTK_OBJECT (frmComposer));
  gtk_widget_set_name (cmdComposerEncrypt, "cmdComposerEncrypt");
  gtk_object_set_data (GTK_OBJECT (frmComposer), "cmdComposerEncrypt", cmdComposerEncrypt);
#endif

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbox2, "vbox2");
  gtk_widget_ref (vbox2);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "vbox2", vbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (vbox1), vbox2, TRUE, TRUE, 0);

  notebook = gtk_notebook_new ();
  gtk_widget_set_name (notebook, "notebook");
  gtk_widget_ref (notebook);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "notebook", notebook,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (notebook);
  gtk_box_pack_start (GTK_BOX (vbox2), notebook, FALSE, TRUE, 0);
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_BOTTOM);
  gtk_notebook_set_tab_hborder (GTK_NOTEBOOK (notebook), 0);
  gtk_notebook_set_tab_vborder (GTK_NOTEBOOK (notebook), 0);

  table1 = gtk_table_new (4, 2, FALSE);
  gtk_widget_set_name (table1, "table1");
  gtk_widget_ref (table1);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "table1", table1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table1);
  gtk_container_add (GTK_CONTAINER (notebook), table1);
  gtk_container_set_border_width (GTK_CONTAINER (table1), 2);
  gtk_table_set_col_spacings (GTK_TABLE (table1), 5);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbox3, "hbox3");
  gtk_widget_ref (hbox3);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "hbox3", hbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox3);
  gtk_table_attach (GTK_TABLE (table1), hbox3, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  txtTo = gtk_entry_new ();
  if (recipient != NULL)
     gtk_entry_set_text(GTK_ENTRY(txtTo), (gchar*) recipient);
  gtk_widget_set_name (txtTo, "txtTo");
  gtk_widget_ref (txtTo);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "txtTo", txtTo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txtTo);
  gtk_box_pack_start (GTK_BOX (hbox3), txtTo, TRUE, TRUE, 0);

  cmdToAddrBook = gtk_button_new_with_label (_("..."));
  gtk_widget_set_name (cmdToAddrBook, "cmdToAddrBook");
  gtk_widget_ref (cmdToAddrBook);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "cmdToAddrBook", cmdToAddrBook,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (cmdToAddrBook), "clicked",
                             GTK_SIGNAL_FUNC (on_cmdComposerAddrBook_clicked),
                             GTK_OBJECT (txtTo));
  gtk_widget_show (cmdToAddrBook);
  gtk_box_pack_start (GTK_BOX (hbox3), cmdToAddrBook, FALSE, FALSE, 0);
  GTK_WIDGET_UNSET_FLAGS (cmdToAddrBook, GTK_CAN_FOCUS);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbox4, "hbox4");
  gtk_widget_ref (hbox4);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "hbox4", hbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox4);
  gtk_table_attach (GTK_TABLE (table1), hbox4, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  txtCC = gtk_entry_new ();
  if (cc != NULL)
     gtk_entry_set_text(GTK_ENTRY(txtCC), (gchar*) cc);
  gtk_widget_set_name (txtCC, "txtCC");
  gtk_widget_ref (txtCC);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "txtCC", txtCC,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txtCC);
  gtk_box_pack_start (GTK_BOX (hbox4), txtCC, TRUE, TRUE, 0);

  cmdCCAddrBook = gtk_button_new_with_label (_("..."));
  gtk_widget_set_name (cmdCCAddrBook, "cmdCCAddrBook");
  gtk_widget_ref (cmdCCAddrBook);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "cmdCCAddrBook", cmdCCAddrBook,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (cmdCCAddrBook), "clicked",
                             GTK_SIGNAL_FUNC (on_cmdComposerAddrBook_clicked),
                             GTK_OBJECT (txtCC));
  gtk_widget_show (cmdCCAddrBook);
  gtk_box_pack_start (GTK_BOX (hbox4), cmdCCAddrBook, FALSE, FALSE, 0);
  GTK_WIDGET_UNSET_FLAGS (cmdCCAddrBook, GTK_CAN_FOCUS);

  hbox5 = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbox5, "hbox5");
  gtk_widget_ref (hbox5);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "hbox5", hbox5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox5);
  gtk_table_attach (GTK_TABLE (table1), hbox5, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  txtBCC = gtk_entry_new ();
  if (bcc != NULL)
     gtk_entry_set_text(GTK_ENTRY(txtBCC), (gchar*) bcc);
  gtk_widget_set_name (txtBCC, "txtBCC");
  gtk_widget_ref (txtBCC);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "txtBCC", txtBCC,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txtBCC);
  gtk_box_pack_start (GTK_BOX (hbox5), txtBCC, TRUE, TRUE, 0);

  cmdBCCAddrBook = gtk_button_new_with_label (_("..."));
  gtk_widget_set_name (cmdBCCAddrBook, "cmdBCCAddrBook");
  gtk_widget_ref (cmdBCCAddrBook);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "cmdBCCAddrBook", cmdBCCAddrBook,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (cmdBCCAddrBook), "clicked",
                             GTK_SIGNAL_FUNC (on_cmdComposerAddrBook_clicked),
                             GTK_OBJECT (txtBCC));
  gtk_widget_show (cmdBCCAddrBook);
  gtk_box_pack_start (GTK_BOX (hbox5), cmdBCCAddrBook, FALSE, FALSE, 0);
  GTK_WIDGET_UNSET_FLAGS (cmdBCCAddrBook, GTK_CAN_FOCUS);

  cmboFrom = gtk_combo_new ();
  gtk_widget_set_name (cmboFrom, "cmboFrom");
  gtk_widget_ref (cmboFrom);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "cmboFrom", cmboFrom,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cmboFrom);
  gtk_table_attach (GTK_TABLE (table1), cmboFrom, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  txtFrom = GTK_COMBO (cmboFrom)->entry;
  if (sender != NULL)
     gtk_entry_set_text(GTK_ENTRY(txtFrom), (gchar*) sender);
  else
     gtk_entry_set_text(GTK_ENTRY(txtFrom), (gchar*) SenderAddr);
  gtk_widget_set_name (txtFrom, "txtFrom");
  gtk_widget_ref (txtFrom);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "txtFrom", txtFrom,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txtFrom);

  lblTo = gtk_label_new (_("To"));
  gtk_widget_set_name (lblTo, "lblTo");
  gtk_widget_ref (lblTo);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "lblTo", lblTo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lblTo);
  gtk_table_attach (GTK_TABLE (table1), lblTo, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (lblTo), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (lblTo), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (lblTo), 2, 0);

  lblFrom = gtk_label_new (_("From"));
  gtk_widget_set_name (lblFrom, "lblFrom");
  gtk_widget_ref (lblFrom);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "lblFrom", lblFrom,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lblFrom);
  gtk_table_attach (GTK_TABLE (table1), lblFrom, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (lblFrom), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (lblFrom), 7.45058e-09, 0.5);
  gtk_misc_set_padding (GTK_MISC (lblFrom), 2, 0);

  lblCC = gtk_label_new (_("CC"));
  gtk_widget_set_name (lblCC, "lblCC");
  gtk_widget_ref (lblCC);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "lblCC", lblCC,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lblCC);
  gtk_table_attach (GTK_TABLE (table1), lblCC, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (lblCC), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (lblCC), 7.45058e-09, 0.5);
  gtk_misc_set_padding (GTK_MISC (lblCC), 2, 0);

  lblBCC = gtk_label_new (_("BCC"));
  gtk_widget_set_name (lblBCC, "lblBCC");
  gtk_widget_ref (lblBCC);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "lblBCC", lblBCC,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lblBCC);
  gtk_table_attach (GTK_TABLE (table1), lblBCC, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (lblBCC), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (lblBCC), 7.45058e-09, 0.5);
  gtk_misc_set_padding (GTK_MISC (lblBCC), 2, 0);

  lblAddress = gtk_label_new (_("Address"));
  gtk_widget_set_name (lblAddress, "lblAddress");
  gtk_widget_ref (lblAddress);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "lblAddress", lblAddress,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lblAddress);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 0), lblAddress);

  scrollAttachments = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_name (scrollAttachments, "scrollAttachments");
  gtk_widget_ref (scrollAttachments);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "scrollAttachments", scrollAttachments,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrollAttachments);
  gtk_container_add (GTK_CONTAINER (notebook), scrollAttachments);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollAttachments), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);

  clistAttachments = gtk_clist_new (2);
  gtk_widget_set_name (clistAttachments, "clistAttachments");
  gtk_widget_ref (clistAttachments);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "clistAttachments", clistAttachments,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (clistAttachments), "button_press_event",
                             GTK_SIGNAL_FUNC (on_clistAttachments_button_press_event),
                             GTK_OBJECT (clistAttachments));
  gtk_widget_show (clistAttachments);
  gtk_container_add (GTK_CONTAINER (scrollAttachments), clistAttachments);
  gtk_clist_set_column_width (GTK_CLIST (clistAttachments), 0, 80);
  gtk_clist_set_column_width (GTK_CLIST (clistAttachments), 1, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (clistAttachments));

  lblSize = gtk_label_new (_("Size"));
  gtk_widget_set_name (lblSize, "lblSize");
  gtk_widget_ref (lblSize);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "lblSize", lblSize,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lblSize);
  gtk_clist_set_column_widget (GTK_CLIST (clistAttachments), 0, lblSize);

  lblFileName = gtk_label_new (_("File Name"));
  gtk_widget_set_name (lblFileName, "lblFileName");
  gtk_widget_ref (lblFileName);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "lblFileName", lblFileName,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lblFileName);
  gtk_clist_set_column_widget (GTK_CLIST (clistAttachments), 1, lblFileName);

  lblAttachments = gtk_label_new (_("Attachments"));
  gtk_widget_set_name (lblAttachments, "lblAttachments");
  gtk_widget_ref (lblAttachments);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "lblAttachments", lblAttachments,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lblAttachments);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 1), lblAttachments);

  hbox6 = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbox6, "hbox6");
  gtk_widget_ref (hbox6);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "hbox6", hbox6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox6);
  gtk_container_add (GTK_CONTAINER (notebook), hbox6);

  vbox4 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbox4, "vbox4");
  gtk_widget_ref (vbox4);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "vbox4", vbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox4);
  gtk_box_pack_start (GTK_BOX (hbox6), vbox4, TRUE, TRUE, 0);

  lblOptions = gtk_label_new (_("Options"));
  gtk_widget_set_name (lblOptions, "lblOptions");
  gtk_widget_ref (lblOptions);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "lblOptions", lblOptions,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lblOptions);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 2), lblOptions);

  hbox2 = gtk_hbox_new (FALSE, 10);
  gtk_widget_set_name (hbox2, "hbox2");
  gtk_widget_ref (hbox2);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "hbox2", hbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox2, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox2), 2);

  hbox10 = gtk_hbox_new (FALSE, 4);
  gtk_widget_set_name (hbox10, "hbox10");
  gtk_widget_ref (hbox10);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "hbox10", hbox10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox10);
  gtk_box_pack_start (GTK_BOX (hbox2), hbox10, TRUE, TRUE, 0);

  lblSubject = gtk_label_new (_("Subject"));
  gtk_widget_set_name (lblSubject, "lblSubject");
  gtk_widget_ref (lblSubject);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "lblSubject", lblSubject,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lblSubject);
  gtk_box_pack_start (GTK_BOX (hbox10), lblSubject, FALSE, FALSE, 0);
  gtk_misc_set_padding (GTK_MISC (lblSubject), 2, 0);

  txtSubject = gtk_entry_new ();
  if (subject != NULL)
     gtk_entry_set_text(GTK_ENTRY(txtSubject), (gchar*) subject);
  gtk_widget_set_name (txtSubject, "txtSubject");
  gtk_widget_ref (txtSubject);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "txtSubject", txtSubject,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txtSubject);
  gtk_box_pack_start (GTK_BOX (hbox10), txtSubject, TRUE, TRUE, 0);

  hbox9 = gtk_hbox_new (FALSE, 4);
  gtk_widget_set_name (hbox9, "hbox9");
  gtk_widget_ref (hbox9);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "hbox9", hbox9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox9);
  gtk_box_pack_start (GTK_BOX (hbox2), hbox9, FALSE, TRUE, 0);

  lblPriority = gtk_label_new (_("Priority"));
  gtk_widget_set_name (lblPriority, "lblPriority");
  gtk_widget_ref (lblPriority);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "lblPriority", lblPriority,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lblPriority);
  gtk_box_pack_start (GTK_BOX (hbox9), lblPriority, FALSE, FALSE, 0);

  optPriority = gtk_option_menu_new ();
  gtk_widget_set_name (optPriority, "optPriority");
  gtk_widget_ref (optPriority);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "optPriority", optPriority,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (optPriority);
  gtk_box_pack_start (GTK_BOX (hbox9), optPriority, FALSE, FALSE, 0);
  optPriority_menu = gtk_menu_new ();
  glade_menuitem = gtk_menu_item_new_with_label (_("Highest"));
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optPriority_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("High"));
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optPriority_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("Normal"));
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optPriority_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("Low"));
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optPriority_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("Lowest"));
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optPriority_menu), glade_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (optPriority), optPriority_menu);
  gtk_option_menu_set_history (GTK_OPTION_MENU (optPriority), 2);

  frame2 = gtk_frame_new (NULL);
  gtk_widget_set_name (frame2, "frame2");
  gtk_widget_ref (frame2);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "frame2", frame2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame2);
  gtk_box_pack_start (GTK_BOX (vbox2), frame2, TRUE, TRUE, 0);

  scrollBody = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_name (scrollBody, "scrollBody");
  gtk_widget_ref (scrollBody);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "scrollBody", scrollBody,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrollBody);
  gtk_container_add (GTK_CONTAINER (frame2), scrollBody);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollBody), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

  /* Load a fixed font. */
  if (mesg_body_font != NULL)
     fixed_font = gdk_font_load (mesg_body_font);

  txtBody = gtk_text_new (NULL, NULL);
  if (body != NULL)
     gti (GTK_TEXT (txtBody), fixed_font, NULL, NULL, (gchar *)body, -1);
  else
     gti (GTK_TEXT (txtBody), fixed_font, NULL, NULL, (gchar *)"\n", -1);
  
  gtk_text_set_point(GTK_TEXT(txtBody), 0);
  gtk_text_set_word_wrap (GTK_TEXT(txtBody), TRUE);
  gtk_editable_set_position (GTK_EDITABLE(txtBody), 0);
  gtk_widget_set_name (txtBody, "txtBody");
  gtk_widget_ref (txtBody);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "txtBody", txtBody,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (txtBody);
  gtk_container_add (GTK_CONTAINER (scrollBody), txtBody);
  gtk_text_set_editable (GTK_TEXT (txtBody), TRUE);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbox1, "hbox1");
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, TRUE, 0);

  progressComposerSend = gtk_progress_bar_new ();
  gtk_widget_set_name (progressComposerSend, "progressComposerSend");
  gtk_widget_ref (progressComposerSend);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "progressComposerSend", progressComposerSend,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (progressComposerSend);
  gtk_box_pack_start (GTK_BOX (hbox1), progressComposerSend, FALSE, FALSE, 0);
  gtk_progress_bar_set_bar_style (GTK_PROGRESS_BAR (progressComposerSend), GTK_PROGRESS_DISCRETE);
  gtk_progress_set_activity_mode (GTK_PROGRESS (progressComposerSend), TRUE);
  gtk_progress_bar_update(GTK_PROGRESS_BAR(progressComposerSend), 0);

  statusbar = gtk_statusbar_new ();
  gtk_widget_set_name (statusbar, "statusbar");
  gtk_widget_ref (statusbar);
  gtk_object_set_data_full (GTK_OBJECT (frmComposer), "statusbar", statusbar,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (statusbar);
  gtk_box_pack_start (GTK_BOX (hbox1), statusbar, TRUE, TRUE, 0);

  gtk_widget_grab_focus (txtFrom);

  return frmComposer;
}

GtkWidget* create_frmComposeSaveAs ()
{
  GtkWidget *frmComposeSaveAs;
  GtkWidget *cmdComposerSaveAs_OK;
  GtkWidget *cmdComposerSaveAs_Cancel;

  frmComposeSaveAs = gtk_file_selection_new (_("Save Composition As..."));
  gtk_widget_set_name (frmComposeSaveAs, "frmComposeSaveAs");
  gtk_object_set_data (GTK_OBJECT (frmComposeSaveAs), "frmComposeSaveAs", frmComposeSaveAs);
  gtk_container_border_width (GTK_CONTAINER (frmComposeSaveAs), 10);
  gtk_signal_connect_object (GTK_OBJECT (frmComposeSaveAs), "destroy",
                             GTK_SIGNAL_FUNC (on_frmComposeSaveAs_destroy),
                             GTK_OBJECT (frmComposeSaveAs));

  cmdComposerSaveAs_OK = GTK_FILE_SELECTION (frmComposeSaveAs)->ok_button;
  gtk_widget_set_name (cmdComposerSaveAs_OK, "cmdComposerSaveAs_OK");
  gtk_object_set_data (GTK_OBJECT (frmComposeSaveAs), "cmdComposerSaveAs_OK", cmdComposerSaveAs_OK);
  gtk_widget_show (cmdComposerSaveAs_OK);
  GTK_WIDGET_SET_FLAGS (cmdComposerSaveAs_OK, GTK_CAN_DEFAULT);
  gtk_signal_connect_object (GTK_OBJECT (cmdComposerSaveAs_OK), "clicked",
                             GTK_SIGNAL_FUNC (on_cmdComposeSaveAs_OK_clicked),
                             GTK_OBJECT (frmComposeSaveAs));

  cmdComposerSaveAs_Cancel = GTK_FILE_SELECTION (frmComposeSaveAs)->cancel_button;
  gtk_widget_set_name (cmdComposerSaveAs_Cancel, "cmdComposerSaveAs_Cancel");
  gtk_object_set_data (GTK_OBJECT (frmComposeSaveAs), "cmdComposerSaveAs_Cancel", cmdComposerSaveAs_Cancel);
  gtk_widget_show (cmdComposerSaveAs_Cancel);
  GTK_WIDGET_SET_FLAGS (cmdComposerSaveAs_Cancel, GTK_CAN_DEFAULT);
  gtk_signal_connect_object (GTK_OBJECT (cmdComposerSaveAs_Cancel), "clicked",
                             GTK_SIGNAL_FUNC (on_cmdComposeSaveAs_Cancel_clicked),
                             GTK_OBJECT (frmComposeSaveAs));

  return frmComposeSaveAs;
}

GtkWidget *create_mnuAttachments(GtkWidget *widget)
{
  GtkWidget *mnuAttachments_menu;
  /*GtkWidget *mnuAttachments;*/
  GtkWidget *mnuAttachmentsAttach;
  GtkWidget *mnuAttachmentsRemove;
  /*GtkWidget *mnuAttachmentsSave;*/

  mnuAttachments_menu = gtk_menu_new ();
  gtk_widget_set_name (mnuAttachments_menu, "mnuAttachments_menu");
  gtk_object_set_data (GTK_OBJECT (frmComposer), "mnuAttachments_menu", mnuAttachments_menu);
  /*gtk_menu_item_set_submenu (GTK_MENU_ITEM (mnuAttachments), mnuAttachments_menu);*/

  mnuAttachmentsAttach = gtk_menu_item_new_with_label (_("Attach..."));
  gtk_widget_set_name (mnuAttachmentsAttach, "mnuAttachmentsAttach");
  gtk_object_set_data (GTK_OBJECT (frmComposer), "mnuAttachmentsAttach", mnuAttachmentsAttach);
  gtk_widget_show (mnuAttachmentsAttach);
  gtk_container_add (GTK_CONTAINER (mnuAttachments_menu), mnuAttachmentsAttach);
  gtk_signal_connect_object (GTK_OBJECT (mnuAttachmentsAttach), "activate",
                      GTK_SIGNAL_FUNC (on_mnuAttachmentsAttach_activate), GTK_OBJECT(widget));

  mnuAttachmentsRemove = gtk_menu_item_new_with_label (_("Remove"));
  gtk_widget_set_name (mnuAttachmentsRemove, "mnuAttachmentsRemove");
  gtk_object_set_data (GTK_OBJECT (frmComposer), "mnuAttachmentsRemove", mnuAttachmentsRemove);
  gtk_widget_show (mnuAttachmentsRemove);
  gtk_container_add (GTK_CONTAINER (mnuAttachments_menu), mnuAttachmentsRemove);
  gtk_signal_connect (GTK_OBJECT (mnuAttachmentsRemove), "activate",
                      GTK_SIGNAL_FUNC (on_mnuAttachmentsRemove_activate), GTK_OBJECT(widget));

  return mnuAttachments_menu;
}

/* frmComposeSaveAs file selector callbacks */
void on_frmComposeSaveAs_destroy (GtkWidget *widget, gpointer user_data)
{
   gtk_widget_destroy(widget);
   SaveAsOpen = FALSE;
}

void on_cmdComposeSaveAs_OK_clicked (GtkWidget *widget, gpointer user_data)
{
   Email message;

   if (SaveFile != NULL)
      g_free(SaveFile);

   SaveFile = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(widget)));
   composer_save_email(frmComposer, &message, SaveFile, "wt");

   gtk_widget_destroy(widget);
   SaveAsOpen = FALSE;
}

void on_cmdComposeSaveAs_Cancel_clicked (GtkWidget *widget, gpointer user_data)
{
   gtk_widget_destroy(widget);
   SaveAsOpen = FALSE;
}


/* frmComposer callbacks */
void on_frmComposer_show (GtkWidget *widget, gpointer user_data)
{
   if (auto_attach_sig)
   {
      if (use_rand_sig)
         attach_sig(widget);
      else
         attach_sigfile(widget);
   }
}

void on_frmComposer_destroy (GtkWidget *widget, gpointer user_data)
{
   g_list_free (mime_send_parts);
   mime_send_parts = NULL;
   
   gtk_widget_destroy(widget);
   ComposerOpen = FALSE;
}

void on_cmdComposerAddrBook_clicked (GtkWidget *txtField, gpointer user_data)
{
   if (!AddrBookOpen)
   {
      frmAddrBook = create_frmAddrBook(txtField);
      gtk_widget_show(frmAddrBook);
      AddrBookOpen = TRUE;
   }
}

void on_mnuComposerFileSend_activate (GtkWidget *widget, gpointer user_data)
{
   on_cmdComposerSend_clicked (widget, user_data);
}

void on_cmdComposerSend_clicked (GtkWidget *widget, gpointer data)
{
   Email message;
   gchar reason[600];
   gchar *mailbox;

   memset((void*)&message, 0, sizeof(Email));
   message.status = UNDELIVERED;

   if (!prepare_message(widget, &message))
   {
      email_destroy(&message);
      return;
   }

   fprintf(stderr, "Sending message...");
   fflush(stderr);

   send_message(&SmtpServer, &message);   
	
   /* if status is DELIVERED, set it to NULL. 
    * if status is DELIVERING, set it to UNDELIVERED */
   if ( (message.status = (message.status ^ DELIVERED)) )
   {
      smtperror(reason, sizeof(reason)-1);
      fprintf(stderr, "failed.\nSpruce%s\n", reason);
      fflush(stdout);
      return;
   }
	
   fprintf(stderr, "sent.\n");
   fflush(stderr);

   if (save_outgoing)
   {
      mailbox = SentMailbox();
      composer_save_email(widget, &message, mailbox, "at");
      g_free(mailbox);
   }	

   email_destroy(&message);
   
   gtk_widget_destroy(widget);
}

void on_mnuComposerFileSave_activate (GtkWidget *widget, gpointer user_data)
{
   on_cmdComposerSave_clicked(widget, user_data);
}

void on_cmdComposerSave_clicked (GtkWidget *widget, gpointer data)
{
   Email message;

   if (!SaveAsOpen)
   {
      if (SaveFile == NULL)
      {
         frmComposeSaveAs = create_frmComposeSaveAs();
         gtk_widget_show(frmComposeSaveAs);
         SaveAsOpen = TRUE;
      }
      else
      {
	      composer_save_email(widget, &message, SaveFile, "wt");
      }
   }
}

void on_mnuComposerFileSave_As_activate (GtkWidget *widget, gpointer user_data)
{
   if (!SaveAsOpen)
   {
      frmComposeSaveAs = create_frmComposeSaveAs();
      gtk_widget_show(frmComposeSaveAs);
      SaveAsOpen = TRUE;
   }
}

void on_mnuComposerFilePrint_activate (GtkWidget *widget, gpointer user_data)
{
   /* TODO: Add code to print... */
}

void on_cmdComposerPrint_clicked (GtkWidget *widget, gpointer data)
{
   /* TODO: Add code to print... */
}

void on_mnuComposerFileClose_activate (GtkWidget *widget, gpointer user_data)
{
   gtk_widget_destroy(widget);
}

void on_mnuComposerEditCut_activate (GtkWidget *widget, gpointer user_data)
{
   GtkWidget *txtBody = get_widget(widget, "txtBody");

   gtk_editable_cut_clipboard(GTK_EDITABLE(txtBody));
}

void on_mnuComposerEditCopy_activate (GtkWidget *widget, gpointer user_data)
{
   GtkWidget *txtBody = get_widget(widget, "txtBody");

   gtk_editable_copy_clipboard(GTK_EDITABLE(txtBody));
}

void on_mnuComposerEditPaste_activate (GtkWidget *widget, gpointer user_data)
{
   GtkWidget *txtBody = get_widget(widget, "txtBody");

   gtk_editable_paste_clipboard(GTK_EDITABLE(txtBody));
}

void file_insert_ok (GtkWidget *w, GtkFileSelection *fs)
{
   GtkWidget *txtBody = get_widget(frmComposer, "txtBody");
   GdkFont *fixed_font = NULL;
   gchar buffer[512];
   guint pos;
   FILE *fp;

   fp = fopen(gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)), "rt");
   if (fp == NULL)
   {
      fprintf(stderr, "Spruce: %s not found.\n", gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
      return;
   }

   /* Load a fixed font */
   if (mesg_body_font != NULL)
      fixed_font = gdk_font_load (mesg_body_font);

   pos = gtk_editable_get_position(GTK_EDITABLE(txtBody));

   gtk_text_freeze(GTK_TEXT(txtBody));
   gtk_text_set_point(GTK_TEXT(txtBody), pos);
   while (!feof(fp))
   {
      memset(buffer, 0, sizeof(buffer));
      fgets(buffer, sizeof(buffer)-1, fp);
      gti (GTK_TEXT (txtBody), fixed_font, &txtBody->style->black, NULL, buffer, -1);
   }
   gtk_text_thaw(GTK_TEXT(txtBody));
   fclose(fp);

   gtk_widget_destroy (GTK_WIDGET(fs));
}

void on_mnuComposerInsertFile_activate (GtkWidget *widget, gpointer user_data)
{
   GtkWidget *filew = gtk_file_selection_new (_("Select file to insert"));

   gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->ok_button), "clicked", 
                      GTK_SIGNAL_FUNC(file_insert_ok), filew);
   gtk_signal_connect_object(GTK_OBJECT (GTK_FILE_SELECTION(filew)->cancel_button), "clicked",
                             GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(filew));
   gtk_widget_show(filew);
}

void on_fileselAttachmentOK_clicked (GtkWidget *button, GtkWidget *filesel)
{
	gchar *file;
	gchar *filename;
	GtkWidget *clistMime = get_widget(frmComposer, "clistAttachments");
	gchar *text[2];
	gchar size[32];
	struct stat st;

	file = gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
	
	filename = g_malloc0(strlen(file) + 1);
	strcpy (filename, file);

	if (stat(file, &st) < 0)
		perror("stat");
	
	sprintf(size, "%d", (gint)st.st_size);

	text[1] = file;
	text[0] = size;
	
	gtk_clist_append (GTK_CLIST (clistMime), text);
	
	gtk_widget_destroy (filesel);
}

void on_mnuComposerInsertAttachment_activate (GtkWidget *widget, gpointer user_data)
{
	on_cmdComposerAttach_clicked (widget, user_data);
}

void on_cmdComposerAttach_clicked (GtkWidget *widget, gpointer data)
{
	GtkWidget *filesel;

	filesel = gtk_file_selection_new (_("Filename"));
	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button), "clicked",
                       GTK_SIGNAL_FUNC(on_fileselAttachmentOK_clicked), filesel);
	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button), "clicked",
                       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(filesel));

	gtk_widget_show (filesel);
}

void on_mnuComposerInsertSignature_activate (GtkWidget *widget, gpointer user_data)
{
   GtkWidget *txtBody = get_widget(widget, "txtBody");
   GdkFont *fixed_font = NULL;
   gchar buffer[512];
   FILE *fp;

   fp = fopen(signaturefile, "rt");
   if (fp == NULL)
   {
      fprintf(stderr, "Spruce: %s not found.\n", signaturefile);
      return;
   }

   /* Load a fixed font */
   if (mesg_body_font != NULL)
      fixed_font = gdk_font_load (mesg_body_font);

	gtk_text_freeze(GTK_TEXT(txtBody));

   gtk_text_set_point(GTK_TEXT (txtBody), gtk_text_get_length(GTK_TEXT(txtBody)));
   gti (GTK_TEXT (txtBody), fixed_font, &txtBody->style->black,
                              NULL, "\n-- \n", -1);
 
   while (!feof(fp))
	{
      memset(buffer, 0, sizeof(buffer));
      fgets(buffer, sizeof(buffer)-1, fp);
      gtk_text_set_point(GTK_TEXT (txtBody), gtk_text_get_length(GTK_TEXT(txtBody)));
      gti (GTK_TEXT (txtBody), fixed_font, &txtBody->style->black,
                                 NULL, buffer, -1);
	}

   gtk_text_thaw(GTK_TEXT(txtBody));

   fclose(fp);
}

void on_mnuComposerInsertSpruce_Sig_activate (GtkWidget *widget, gpointer user_data)
{
   GtkWidget *txtBody = get_widget(widget, "txtBody");
   GdkFont *fixed_font = NULL;
   gchar buffer[512];
   gchar *sig_file = get_randsig_path();
   long line, numlines;
   FILE *fp;
	
   srand((unsigned) time(0));   /* randomize the seed */
	
   fp = fopen(sig_file, "rt");
   if (fp == NULL)
   {
      fp = fopen("/etc/sprucesig", "rt");
      if (fp == NULL)
		{
         fprintf(stderr, "Could not attach random signature: %s not found.\n", sig_file);
         return;
		}
   }

   for (numlines = 0; !feof(fp); numlines++)
      fgets(buffer, sizeof(buffer)-1, fp);

   line = rand() % numlines;
   fseek(fp, 0, SEEK_SET);       /* rewind to beginning of file */

   /* Load a fixed font */
   if (mesg_body_font != NULL)
      fixed_font = gdk_font_load (mesg_body_font);
	
   for (numlines = 0; numlines < line; numlines++)
      fgets(buffer, sizeof(buffer)-1, fp);

   gtk_text_freeze(GTK_TEXT(txtBody));
   gtk_text_set_point(GTK_TEXT (txtBody), gtk_text_get_length(GTK_TEXT(txtBody)));
   gti (GTK_TEXT (txtBody), fixed_font, NULL, NULL, "\n-- \n", -1);
   gti (GTK_TEXT (txtBody), fixed_font, NULL, NULL, buffer, -1);
   gtk_text_thaw(GTK_TEXT(txtBody));

   fclose(fp);

   g_free(sig_file);
}

void on_mnuComposerHelpAbout_activate (GtkWidget *widget, gpointer user_data)
{
   if (!AboutOpen)
	{
      frmAbout = create_frmAbout();
      gtk_widget_show(frmAbout);
      AboutOpen = TRUE;
	}
}

void on_clistAttachments_button_press_event (GtkWidget *clist, GdkEvent *event, gpointer user_data)
{
   GtkWidget *menu;
   GtkWidget *widget = (GtkWidget*)user_data;

   if (event->button.button == 3)
   {
      menu = create_mnuAttachments(widget);
      gtk_menu_popup((GtkMenu *)menu, NULL, NULL, NULL,
                     (gpointer)NULL, (guint)NULL, (guint32)NULL);
   }
}


/* mnuAttachments callbacks */
void on_mnuAttachmentsAttach_activate (GtkWidget *widget, gpointer user_data)
{
	GtkWidget *filesel;

	filesel = gtk_file_selection_new (_("Select file to attach..."));
	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button), "clicked",
                       GTK_SIGNAL_FUNC(on_fileselAttachmentOK_clicked), filesel);
	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button), "clicked",
                       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(filesel));

	gtk_widget_show (filesel);
}

void on_mnuAttachmentsRemove_activate (GtkWidget *widget, gpointer user_data)
{
   GtkWidget *clist = get_widget(frmComposer, "clistAttachments");
   GList *selection;
   gint i;

   selection = GTK_CLIST(clist)->selection;

   gtk_clist_freeze(GTK_CLIST(clist));
   i = g_list_length(selection);
   while (selection != NULL && i)
   {
      gtk_clist_remove(GTK_CLIST(clist), (gint)selection->data);
      selection = selection->next;
      i--;
   }
   gtk_clist_thaw(GTK_CLIST(clist));
}


/* other functions */
gint prepare_message(GtkWidget *widget, Email *mesg)
{
   GtkWidget *txtFrom    = get_widget(widget, "txtFrom");
   GtkWidget *txtTo      = get_widget(widget, "txtTo");
   GtkWidget *txtCC      = get_widget(widget, "txtCC");
   GtkWidget *txtBCC     = get_widget(widget, "txtBCC");
   GtkWidget *txtSubject = get_widget(widget, "txtSubject");
   GtkWidget *txtBody    = get_widget(widget, "txtBody");
   GtkWidget *clist      = get_widget(widget, "clistAttachments");
   gchar *buffer = NULL;
   gchar *boundary = NULL, *body;
   gint index;
   gchar addr[256];
   gint fields, rows, i, retval = 1;
   GList *tmp;
 
   if (mesg->status & UNDELIVERED)
   {
      /* Setup the X-Mailer information. */
      mesg->xmailer = g_strdup(xmailer);

      /* Setup the From field of the mesg */
      buffer = make_8bit((unsigned char*)gtk_entry_get_text(GTK_ENTRY(txtFrom)));
      trim_whtspc(buffer);
      if (!buffer || !*buffer)
      {
         /* This can be considered a bad thing ;-)
          * Since this might have been called by a "save" type function, we
          * need to continue onward to obtain all the info that we *do* have so
          * lets make sure we return an error when we're done */
         retval = 0;
      }
      mesg->from = g_strdup(buffer);
      g_free(buffer);

      /* Setup the To field of the mesg */
      buffer = make_8bit((unsigned char*)gtk_entry_get_text(GTK_ENTRY(txtTo)));
      trim_whtspc(buffer);
      if (!buffer || !*buffer)
      {
         /* This can be considered a bad thing ;-)
          * Since this might have been called by a "save" type function, we
          * need to continue onward to obtain all the info that we *do* have so
          * lets make sure we return an error when we're done */
         retval = 0;
      }
      mesg->to = g_strdup(buffer);
      g_free(buffer);
      buffer = NULL;
	
      /* Setup the CC field of the mesg */
      gtk_editable_set_position(GTK_EDITABLE(txtCC), -1);
      if (gtk_editable_get_position(GTK_EDITABLE(txtCC)) > 1)
      {
         buffer = gtk_editable_get_chars(GTK_EDITABLE(txtCC), 0, -1);
         trim_whtspc(buffer);
         if (buffer == (gchar*)NULL)
            mesg->cc = NULL;
         else
         {
            mesg->cc = make_8bit((unsigned char*)buffer);
            g_free(buffer);
            buffer = NULL;
         }
      }
      else
         mesg->cc = NULL;

      /* Setup the BCC field of the mesg. */
      gtk_editable_set_position(GTK_EDITABLE(txtBCC), -1);
      if (gtk_editable_get_position(GTK_EDITABLE(txtBCC)) > 1)
      {
         buffer = gtk_editable_get_chars(GTK_EDITABLE(txtBCC), 0, -1);
         trim_whtspc(buffer);
         if (buffer == (gchar*)NULL)
            mesg->bcc = NULL;
         else
         {
            mesg->bcc = make_8bit((unsigned char*)buffer);
            g_free(buffer);
            buffer = NULL;
         }
      }
      else
         mesg->bcc = NULL;

      /* Setup the subject of the mesg. */
      buffer = make_8bit((unsigned char*)gtk_entry_get_text(GTK_ENTRY(txtSubject)));
      trim_whtspc(buffer);
      mesg->subject = g_strdup(buffer);
      g_free(buffer);

      /* populate the mime_send_parts GList based on clistAttachments */
      g_list_free(mime_send_parts);
      rows = GTK_CLIST(clist)->rows;
      for (i = 0; i < rows; i++)
      {
         gtk_clist_get_text(GTK_CLIST(clist), i, 1, &buffer);
         mime_send_parts = g_list_append(mime_send_parts, g_strdup(buffer));
      }

      /* Set the body of the message into the structure */
      if (g_list_length (mime_send_parts) > 0)
      {
         boundary = g_strdup(mime_get_boundary());
         body = g_malloc0(210 + strlen(boundary) + strlen(gtk_editable_get_chars(GTK_EDITABLE(txtBody), 0, -1)));
         index = 0;
         index += sprintf(body, "This is a multi-part message in MIME format.\n\n");
         buffer = g_strdup(gtk_editable_get_chars(GTK_EDITABLE(txtBody), 0, -1));
         index += sprintf(body+index, "--%s\n", boundary);
         index += sprintf(body+index, "Content-Type: text/plain\nContent-Transfer-Type: 8bit\n\n%s\n", word_wrap(buffer, word_wrap_len));
         g_free(buffer);
         tmp = mime_send_parts;
         while (tmp != NULL)
         {
            index += mime_insert_part (&body, index, boundary, (gchar *)tmp->data);
            tmp = tmp->next;
         }
         body = g_realloc(body, strlen(body) + strlen(boundary) + 7);
         /* insert the end marking boundary */
         sprintf(body + index, "--%s--\n", boundary);
         body[index + 2 + strlen(boundary) + 3] = '\0';
         mesg->body = g_strdup(body);
         g_free(body);
      }
      else
      {
         mesg->body = word_wrap(g_strdup(gtk_editable_get_chars(GTK_EDITABLE(txtBody), 0, -1)), word_wrap_len);
      }

      /* Set sender's email address. */
      get_email_addr_from_text(addr, mesg->from);
      mesg->from_addr = g_strdup(addr);

      /* Set CC email address. */
      parse_addr_list(mesg->cc);
      mesg->cc_addr = (gchar*) NULL;
      fields = get_num_fields(mesg->cc, ';');
      for (i = 1; i <= fields; i++)
      {
         buffer = get_field(mesg->cc, i, ';');
         memset(addr, 0, sizeof(addr));
         get_email_addr_from_text(addr, buffer);
         g_free(buffer);
         if (i == 1)
         {
            mesg->cc_addr = g_strdup(addr);
         }
         else
         {
            mesg->cc_addr = g_realloc(mesg->cc_addr, strlen(mesg->cc_addr) + strlen(addr) + 2);
            strcat(mesg->cc_addr, ",");
            strcat(mesg->cc_addr, addr);
         }
      }
      unparse_addr_list(mesg->cc);

      /* Set BCC email address. */
      parse_addr_list(mesg->bcc);
      mesg->bcc_addr = (gchar*) NULL;
      fields = get_num_fields(mesg->bcc, ';');
      for (i = 1; i <= fields; i++)
      {
         buffer = get_field(mesg->bcc, i, ';');
         memset(addr, 0, sizeof(addr));
         get_email_addr_from_text(addr, buffer);
         g_free(buffer);
         if (i == 1)
         {
            mesg->bcc_addr = g_strdup(addr);
         }
         else
         {
            mesg->bcc_addr = g_realloc(mesg->bcc_addr, strlen(mesg->bcc_addr) + strlen(addr) + 2);
            strcat(mesg->bcc_addr, ",");
            strcat(mesg->bcc_addr, addr);
         }
      }
      unparse_addr_list(mesg->bcc);

      /* setup the mime stuff */
      if (g_list_length (mime_send_parts) > 0)
      {
         mesg->mime_version = g_strdup ("1.0");
         buffer = g_malloc0(sizeof(char)*76);
         sprintf(buffer, "multipart/mixed; boundary=\"%s\"", boundary);
         mesg->content_transfer_encoding = NULL;
         mesg->content_type = g_strdup(buffer);
         g_free(buffer);
      }
      else
      {
         /* for now we just use this, later on we should autodetect
          * if its 7bit or 8bit or if we should use quoted_printable etc.. */
         mesg->mime_version = g_strdup ("1.0");
         mesg->content_transfer_encoding = g_strdup ("8bit");
         mesg->content_type = g_strdup ("text/plain; charset=\"iso-8859-1\"");
      }

      /* setup the Reply-To information... */
      if (ReplyTo != NULL && *ReplyTo != '\0')
         mesg->replyto = g_strdup(ReplyTo);
      else
         mesg->replyto = g_strdup(mesg->from_addr);

      /* Parse for each recipient and add them to the list */
      parse_addr_list(mesg->to);
      fields = get_num_fields(mesg->to, ';');
      for (i = 1; i <= fields; i++)
      {
         buffer = get_field(mesg->to, i, ';');
         memset(addr, 0, sizeof(addr));
         get_email_addr_from_text(addr, buffer);
         g_free(buffer);
         if (i == 1)
         {
            mesg->to_addr = g_strdup(addr);
         }
         else
         {
            mesg->to_addr = g_realloc(mesg->to_addr, strlen(mesg->to_addr) + strlen(addr) + 2);
            strcat(mesg->to_addr, ",");
            strcat(mesg->to_addr, addr);
         }
      }
      unparse_addr_list(mesg->to);

      /* Now that we have the Reply-To addr, we can format the header 
        information... */
      if (!format_header(mesg))
         retval = 0;

      g_free(boundary);
   }

   return retval;
}

gchar* get_randsig_path()
{
   gchar *home = NULL;
   gchar *path = NULL;

   home = getenv("HOME");
   if (!home)
      return NULL;

   path = g_strconcat(home, "/", sprucesig, NULL);

   return path;	
}

gint attach_sig(GtkWidget *widget)
{
   GtkWidget *txtBody = get_widget(widget, "txtBody");
   GdkFont *fixed_font = NULL;
   gchar buffer[513];
   gchar *sig_file = get_randsig_path();
   long line, numlines;
   FILE *fp;
	
   srand((unsigned) time(NULL));   /* randomize the seed */
	
   fp = fopen(sig_file, "rt");
   if(fp == NULL)
   {
      fp = fopen("/etc/sprucesig", "rt");
      if (fp == NULL)
		{
         fprintf(stderr, "Could not attach random signature: %s not found.\n", sig_file);
         return 0;
		}
   }

   for (numlines = 0; !feof(fp); numlines++)
      fgets(buffer, 512, fp);

   line = rand() % numlines;
   fseek(fp, 0, SEEK_SET);       /* rewind to beginning of file */
	
   for (numlines = 0; numlines < line; numlines++)
      fgets(buffer, 512, fp);

   /* Load a fixed font */
   if (mesg_body_font != NULL)
      fixed_font = gdk_font_load (mesg_body_font);

   gtk_text_freeze(GTK_TEXT(txtBody));
   gtk_text_set_point(GTK_TEXT (txtBody), gtk_text_get_length(GTK_TEXT(txtBody)));
   gti (GTK_TEXT (txtBody), fixed_font, NULL, NULL, "\n-- \n", -1);
   gti (GTK_TEXT (txtBody), fixed_font, NULL, NULL, buffer, -1);
   gtk_text_thaw(GTK_TEXT(txtBody));

   fclose(fp);

   g_free(sig_file);
   
   return 1;
}

gint attach_sigfile(GtkWidget *widget)
{
   GtkWidget *txtBody = get_widget(widget, "txtBody");
   GdkFont *fixed_font = NULL;
   gchar buffer[513];
   FILE *fp;

   fp = fopen(signaturefile, "rt");
   if(fp == NULL)
   {
      fprintf(stderr, "Spruce: %s not found.\n", signaturefile);
      return 0;
   }

   gtk_text_freeze(GTK_TEXT(txtBody));

   /* Load a fixed font */
   if (mesg_body_font != NULL)
      fixed_font = gdk_font_load (mesg_body_font);	

   gtk_text_set_point(GTK_TEXT (txtBody), gtk_text_get_length(GTK_TEXT(txtBody)));
   gti (GTK_TEXT (txtBody), fixed_font, &txtBody->style->black, NULL, "\n-- \n", -1);
 
   while (!feof(fp))
	{
      memset(buffer, 0, sizeof(buffer));
      fgets(buffer, sizeof(buffer)-1, fp);
      gtk_text_set_point(GTK_TEXT (txtBody), gtk_text_get_length(GTK_TEXT(txtBody)));
      gti (GTK_TEXT (txtBody), fixed_font, &txtBody->style->black, NULL, buffer, -1);
	}

   gtk_text_thaw(GTK_TEXT(txtBody));

   fclose(fp);
   
   return 0;
}

gint composer_save_email(GtkWidget *widget, Email *mesg, gchar *filename, gchar *mode)
{
   FILE *fp;

   fp = fopen(filename, mode);
   if (fp == NULL)
   {
      fprintf(stderr, "Warning: %s failed for writing.\n", filename);
      return 0;
   }

   if (!strcmp(mode, "wt"))
   {
      /* we are not saving into Sent items... */
      mesg->status = UNDELIVERED;
      prepare_message(widget, mesg);
   }
   else
      format_header(mesg);

   fprintf(fp, "%s\n\n", mesg->header);
   fflush(fp);
   fprintf(fp, "%s", mesg->body);
   fflush(fp);
   fprintf(fp, "\n.\r\n");
   fflush(fp);

   fclose(fp);

   if (!strcmp(mode, "wt"))
   {
      /* we are not saving into Sent items... */
      email_destroy(mesg);
      mesg->status = UNDELIVERED;
   }

   return 1;
}

gchar* SentMailbox()
{
   gchar *home = NULL;
   gchar *path = NULL;

   home = getenv("HOME");
   if (!home)
      return NULL;

   path = g_strconcat(home, "/.spruce/Sent_items.mbx", NULL);

   return path;	
}

#if defined(ENABLE_GPG) || defined (ENABLE_PGP)
void on_cmdComposerEncrypt_clicked(GtkWidget *widget, gpointer data)
{
   Composer_PGP_Encrypt(widget);
}

void Composer_PGP_Encrypt (GtkWidget *widget)
{
   GtkWidget *txtBody = get_widget(frmComposer, "txtBody");
   GtkWidget *txtTo   = get_widget(frmComposer, "txtTo");
   GtkWidget *txtCC   = get_widget(frmComposer, "txtCC");
   GdkFont *fixed_font = NULL;
   gchar recipients[2049];
   gchar recip[256];
   gchar *recip_buffer, *recip_temp, *cc_buffer, *buff;
   gchar *text;
   gchar *pgpd_mesg;
   guint len;
   guint num, i;

   memset(recipients, 0, sizeof(recipients));

   text = gtk_editable_get_chars(GTK_EDITABLE(txtBody), 0, -1);
   recip_buffer = gtk_editable_get_chars(GTK_EDITABLE(txtTo), 0, -1);
   cc_buffer = gtk_editable_get_chars(GTK_EDITABLE(txtCC), 0, -1);

   if (cc_buffer != (gchar *)NULL)
   {
       unsigned int recip_size = strlen(recip_buffer);
       unsigned int cc_size = strlen(cc_buffer);

       if (cc_size > 0)
       {
	      recip_temp = g_strdup(recip_buffer);
          recip_buffer = g_realloc(recip_buffer, recip_size + cc_size + 2);
          g_snprintf(recip_buffer, recip_size + cc_size + 2, "%s;%s", recip_temp, cc_buffer);
       }

       g_free(cc_buffer);
       cc_buffer = (gchar *)NULL;
   }
   
   /* make the recipients list */
   num = get_num_fields(recip_buffer, ';');
   for (i = 1; i <= num; i++)
   {
      buff = get_field(recip_buffer, i, ';');
      get_email_addr_from_text (recip, buff);
	  trim_whtspc(recip);
      if (i > 1)
         strncat(recipients, " ", sizeof(recipients)-1);
      strncat(recipients, recip, sizeof(recipients)-1);
      g_free(buff);
   }
   
   g_free(recip_buffer);

   if (encrypt_to_self)
   {
      strncat(recipients, " ", sizeof(recipients)-1);
      strncat(recipients, pgp_identity, sizeof(recipients)-1);
   }

   pgpd_mesg = pgp_encrypt(text, recipients, use_gnupg, pgp_sign);
   g_free(text);

   if (pgpd_mesg == NULL)
      return;

   /* Load a fixed font */
   if (mesg_body_font != NULL)
      fixed_font = gdk_font_load (mesg_body_font);

   gtk_text_freeze(GTK_TEXT(txtBody));

   /* delete all the the text in there */
   len = gtk_text_get_length(GTK_TEXT(txtBody));
   gtk_text_set_point(GTK_TEXT(txtBody), len);
   gtk_text_backward_delete(GTK_TEXT(txtBody), len);

   gti (GTK_TEXT (txtBody), fixed_font, NULL, NULL, pgpd_mesg, -1);
   g_free(pgpd_mesg);

   gtk_text_thaw(GTK_TEXT(txtBody));
}
#endif /* defined(ENABLE_GPG) || defined (ENABLE_PGP) */
