/*  Spruce
 *  Copyright (C) 1999-2000 Jeffrey Stedfast
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#	include "config.h"
#endif
                       
#include "globals.h"
#include "server.h"
#include "accounts.h"

/* static vars */
static gchar configfile[] = ".spruce/spruce.conf";

/* lets keep track of open windows... */
gboolean AboutOpen;
gboolean AccountsOpen;
gboolean AddrBookOpen;
gboolean ComposerOpen;
gboolean FiltersOpen;
gboolean OptionsOpen;
gboolean PassphraseOpen;

/* generic globals */
gchar *spruceconf;                       /* spruce config file */

gint NumMailboxes;                       /* number of mailboxes (Inbox, Outbox, etc) */

gchar *AddrSelected;                     /* addr selected by frmAddrBook */
gint first_time;
gint texticons;                          /* use text? icons? or both? */
GList *filters;                          /* Filter list */
GList *mime_parts;
GList *mime_send_parts;

/* Identity */
gchar *SenderAddr;                       /* user's email addr */
gchar *ReplyTo;                          /* user's Reply-To email addr */
gint use_rand_sig;                       /* use spruce's random signature? */
gint auto_attach_sig;                    /* auto-attach the signature? */
gchar *signaturefile;                    /* user's signature file */

/* Accounts */
GList *PopAccounts;
GList *ImapAccounts;
Server SmtpServer;                       /* smtp server */
gint external_fetchmail;                 /* use an external fetchmail program */

gint use_rand_sig;                       /* attach a random signature? */
gint save_outgoing;                      /* save outgoing messages? */
gchar *signaturefile;                    /* user's signature file */
Server SmtpServer;                       /* smtp server */

/* Appearance */
gchar *mesg_body_font;
gchar *unread_mesg_header_font;
gchar *read_mesg_header_font;

/* Misc */
gint save_outgoing;                      /* save outgoing messages? */
gint always_prepend;                     /* always prepend Re: or Fw: ?? */
gint word_wrap_len;                      /* chars per line */
gint empty_on_exit;                      /* empty "Deleted items" on exit? */
gint cm_delay;                           /* check mail every XX minutes */
gint hdr_trim_level;                     /* 0 = none; 1 = partial; 2 = full */
gint save_sizes;

/* PGP */
gchar *pgp_identity;                     /* PGP Identity */
gint use_gnupg;                          /* do we use GNUpg or Pgp50i? */
gint pgp_sign;                           /* pgpv -s ? */
gint encrypt_to_self;                    /* always encrypt to self? */
gint store_passphrase;                   /* store our passphrase? */
gchar *pgp_passphrase;                   /* pgp passphrase */

/* Sorting preferences */
gint sort_column;                        /* column to sort */
GtkSortType sort_type;

/* here are our window widgets... */
GtkWidget *frmAbout    = NULL;
GtkWidget *frmAccounts = NULL;
GtkWidget *frmAddrBook = NULL;
GtkWidget *frmComposer = NULL;
GtkWidget *frmFilters  = NULL;
GtkWidget *frmMain     = NULL;
GtkWidget *frmOptions  = NULL;
GtkWidget *frmStatus   = NULL;
GtkWidget *dlgPassphrase = NULL;


gchar *get_config_path()
{
   gchar *home;
   gchar *path;

   home = getenv("HOME");

   if (home == NULL)
      return NULL;

   path = g_strconcat(home, "/", configfile, NULL);

   return path;	
}

void reset_globals()
{
   AboutOpen      = FALSE;
   AccountsOpen   = FALSE;
   AddrBookOpen   = FALSE;
   ComposerOpen   = FALSE;
   FiltersOpen    = FALSE;
   OptionsOpen    = FALSE;
   PassphraseOpen = FALSE;

   NumMailboxes = 0;

   use_rand_sig = TRUE;
   auto_attach_sig = TRUE;
   save_outgoing = TRUE;
   always_prepend = FALSE;
   
   SmtpServer.port = 25;

   external_fetchmail = FALSE;

   spruceconf = get_config_path();

   first_time = 0;

   texticons = TB_ICONS;

   hdr_trim_level = 1;

   empty_on_exit = TRUE;

   word_wrap_len = 76;
   cm_delay = 0;

   ReplyTo = NULL;

   filters = NULL;

   mailpos = NULL;

   use_gnupg = TRUE;
   pgp_sign = FALSE;
   encrypt_to_self = TRUE;
   store_passphrase = FALSE;
   pgp_passphrase = NULL;

   mime_parts = NULL;
   mime_send_parts = NULL;
   mesg_body_font = NULL;

   unread_mesg_header_font = NULL;
   read_mesg_header_font = NULL;

   PopAccounts = NULL;
   ImapAccounts = NULL;
}

void first_run()
{
   gchar *home = NULL;
   gchar *path = NULL;
   
   first_time = 1;

   NumMailboxes = 0;

   if ((home = getenv("HOME")) == NULL)
   {
      if ((home = getenv("USER")) == NULL)
      {
         fprintf(stderr, _("Sorry! Cannot determine where to create the spruce directory!\n"));
         return;	/* What else to do? */
      }
      else 
      {
         path = g_strconcat("/home/", home, "/.spruce", NULL);
      }
   }
   else
   {
      path = g_strconcat(home, "/.spruce", NULL);
   }
   
   mkdir(path, 0700);	/* lets make this dir only readable my the owner! -Chuck */
   g_free(path);

   mailbox_create("Inbox");
   mailbox_create("Drafts");
   mailbox_create("Outbox");
   mailbox_create("Sent items");
   mailbox_create("Deleted items");
}

void global_free()
{
   g_free(spruceconf);
   spruceconf = NULL;
   g_free(SenderAddr);
   SenderAddr = NULL;
   g_free(ReplyTo);
   ReplyTo = NULL;
   g_free(signaturefile);
   signaturefile = NULL;
   g_free(AddrSelected);
   AddrSelected = NULL;
   g_free(mailpos);
   mailpos = NULL;
   g_free(pgp_identity);
   pgp_identity = NULL;
   g_free(pgp_passphrase);
   pgp_passphrase = NULL;
   g_free(mesg_body_font);
   mesg_body_font = NULL;
   g_free(unread_mesg_header_font);
   unread_mesg_header_font = NULL;
   g_free(read_mesg_header_font);
   read_mesg_header_font = NULL;
   null_accounts();
}
