/*  Spruce
 *  Copyright (C) 1999-2000 Jeffrey Stedfast
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "init.h"

extern GtkWidget *frmComposer;
extern gchar *SenderAddr;
extern gint texticons;                          /* use text, icons or both? */

void set_security(void)
{
#if defined(__unix__)
   struct rlimit coredump;

   coredump.rlim_cur = RLIMIT_NOFILE;
   coredump.rlim_max = RLIMIT_NOFILE;
   setrlimit(RLIMIT_CORE, &coredump);  /* we don't want core dumps do we? ;-) */
#endif
}

void spruce_init(int argc, char **argv)
{
   gchar *to = NULL;
   gchar *cc = NULL;
   gchar *bcc = NULL;
   gchar *subject = NULL;
   gint cur = 0, longindex = 0;
   gchar *param = NULL;
   longopts_t longopts[] =
   {
     {"help",     no_arg,       'h'},
     {"version",  no_arg,       'v'},
     {"bindings", no_arg,        1 },
     {"icons",    no_arg,       'i'},
     {"text",     no_arg,       't'},
     {"both",     no_arg,       'o'},
     {"cc",       required_arg, 'c'},
     {"bcc",      required_arg, 'b'},
     {"subject",  required_arg, 's'},
     {"mailto",   required_arg, 'm'},
     {NULL,       no_arg,        0 }
   };

   texticons = 0;

   while (cur != -1)
   {
      cur = getopts_long(argc, argv, "hvito", longopts, &longindex, &param);

      switch (cur)
      {
         case 'h':
            spruce_help(argv[0]);
            break;
         case 'i':
            if (texticons != TB_BOTH && texticons != TB_ICONS)
               texticons += TB_ICONS;
            break;
         case 't':
            if (texticons != TB_BOTH && texticons != TB_TEXT)
               texticons += TB_TEXT;
            break;
         case 'o':
            texticons = TB_BOTH;
            break;
         case 'c':
            if (!cc)
               cc = g_strdup(param);
            break;
         case 'b':
            if (!bcc)
               bcc = g_strdup(param);
            break;
         case 's':
            if (!subject)
               subject = g_strdup(param);
            break;
         case 'm':
            if (!to)
               to = g_strdup(param);
            break;
         case 'v':
            fprintf(stdout, "%s\n", VERSION);
            exit(0);
            break;
         case 1:
            fprintf(stdout, "Key bindings:\n");
            fprintf(stdout, "  Shift + C  \t\tCopy message...\n");
            fprintf(stdout, "  D          \t\tDelete selected message\n");
            fprintf(stdout, "  Control + E\t\tExpunge selected message\n");
            fprintf(stdout, "  F          \t\tForward selected message\n");
            fprintf(stdout, "  Shift + M  \t\tMove message...\n");
            fprintf(stdout, "  M          \t\tNew Message\n");
            fprintf(stdout, "  O          \t\tOpen selected message\n");
            fprintf(stdout, "  Control + P\t\tPrint Message\n");
            fprintf(stdout, "  R          \t\tReply to message\n");
            fprintf(stdout, "  A          \t\tReply To All\n");
            fprintf(stdout, "  Control + S\t\tSave Message As...\n");
            fprintf(stdout, "  Control + Q\t\tExit Spruce\n");
            fprintf(stdout, "  .          \t\tNext message\n");
            fprintf(stdout, "  ,          \t\tPrevious message\n\n");
            fflush(stdout);
            exit(0);
            break;
         default:
            break;
      }
   }
	
   if (!texticons)
      texticons = TB_BOTH;  /* default is toolbar buttons w/ icons and text */

   if (to || cc || bcc || subject)
      frmComposer = create_frmComposer(SenderAddr, to, cc, bcc, subject, NULL);
}

void spruce_help(gchar *program)
{
   fprintf(stdout, "Usage: %s [options ...]\n\n", program);
   fprintf(stdout, "options:\n");
   fprintf(stdout, "  --bindings           show key bindings and exit\n");
   fprintf(stdout, "  -h, --help           output this help and exit\n");
   fprintf(stdout, "  -i, --icons          use icons in toolbars\n");
   fprintf(stdout, "  -t, --text           use text in toolbars\n");
   fprintf(stdout, "  -o, --both           use both text and icons in toolbars\n");
   fprintf(stdout, "  -v, --version        print version information and exit\n\n");
   fprintf(stdout, "special:\n");
   fprintf(stdout, "  --mailto  <address>  specify a mail-to (TO) address\n");
   fprintf(stdout, "  --cc      <address>  specify a carbon-copy (CC) address\n");
   fprintf(stdout, "  --bcc     <address>  specify a blind carbon-copy (BCC) address\n");
   fprintf(stdout, "  --subject <subject>  specify a subject (must be in quotes if it has spaces)\n\n");
   fflush(stdout);
   exit(0);
}
