/*  Spruce
 *  Copyright (C) 1999-2000 Jeffrey Stedfast
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#	include "config.h"
#endif

#include "mbox.h"

#ifndef MAP_FAILED
#define MAP_FAILED NULL
#endif

extern guint32 *mailpos;  /* steal this from mailbox.c */

gboolean mailbox_is_mbox_format (gchar *mbx, gboolean return_last_retval)
{
   static gboolean retval = FALSE;
   gchar buffer[512];
   FILE *fp;

   /* time to see if we want to re-test the format or not */
   if (return_last_retval)
      return retval;  /* return the old return value */

   fp = fopen(mbx, "rt");
   if (fp == NULL)
   {
      retval = FALSE;
      return retval;
   }

   memset(buffer, 0, sizeof(buffer));
   fgets(buffer, sizeof(buffer)-1, fp);
   if (!strncmp("From ", buffer, strlen("From ")))
   {
      fclose(fp);
      retval = TRUE;
      return retval;
   }

   fclose(fp);

   retval = FALSE;
   return retval;
}

gchar *mbox_to_spruce (gchar *mesg)
{
   /* all we really need to do is append "\n.\r\n" */
   gchar *new, *ptr;

   /* trim off the "From " part of the header - aka first line */
   for (ptr = mesg; *ptr && *ptr != '\n'; ptr++);
   ptr++;

   new = g_malloc0(strlen(ptr) + strlen("\n.\r\n") + 1);

   strcpy(new, ptr);
   strcat(new, "\n.\r\n");

   return new;
}

gint mbox_rename (gchar *oldpath, gchar *newpath)
{
   gchar *ptr, *contents;
   struct stat st;
   gint fd1, fd2;

   fd1 = open(oldpath, O_RDONLY);
   if (fd1 == -1)
   {
      perror("open()");
      unlink(oldpath);
      return 0;
   }
   unlink(oldpath);

   if (fstat(fd1, &st) == -1)
   {
      perror("stat()");
      close(fd1);
      return 0;
   }

   if (st.st_size)
   {
      ptr = (gchar*) mmap(0, st.st_size, PROT_READ, MAP_PRIVATE, fd1, 0);
      if (ptr == (gchar*)MAP_FAILED)
      {
         close(fd1);
         perror("mmap()");
         return 0;
      }

      fd2 = open(newpath, O_WRONLY | O_CREAT | O_TRUNC);
      if (fd2 == -1)
      {
         close(fd1);
         munmap(ptr, st.st_size);
         perror("open()");
         return 0;
      }

      contents = g_strdup(ptr);
      write(fd2, contents, strlen(contents));
      g_free(contents);
      close(fd2);
      munmap(ptr, st.st_size);
   }
   else
   {
      fd2 = open(newpath, O_WRONLY | O_CREAT | O_TRUNC);
      if (fd2 == -1)
      {
         close(fd1);
         perror("open()");
         return 0;
      }
      close(fd2);
   }

   close(fd1);

   return 1;
}

static gint mbox_next_after_from (gchar *r)
{
   /* We check for something that looks like a mail header field after
    * the "^From", and for a "\d\d\d\d$" at the end of the from line. */
   gint i;

   if (*r != '\n')
      return 0;
   if (*r < 'A' && *r > 'Z')
      return 0;
   for (i = -1; i > -4; i--)
      if (r[i] > '9' || r[i] < '0')
         return 0;

   r++;
   while (*r && *r != '\n')
   {
      if (*r++ == ':')
         return 1;
   }

   return 0;
}

guint mbox_get_num_mesgs (gchar *mbx)
{
   gchar *p, *q, *r;
   struct stat st;
   gint fd;
   guint num = 0;

   if (mailpos != NULL)
      g_free(mailpos);

   if (mbx == NULL)
      return 0;

   fd = open(mbx, O_RDONLY);
   if (fd == -1)
   {
      perror("open()");
      return 0;
   }

   if (stat(mbx, &st) == -1)
   {
      perror("stat()");
      close(fd);
      return 0;
   }

   if (st.st_size)
   {
      p = q = (gchar*) mmap(0, st.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
      if (p == (gchar*)MAP_FAILED)
      {
         close(fd);
         perror("mmap()");
         return 0;
      }

      if (!strncmp(p, "From ", strlen("From ")))
      {
         mailpos = g_malloc0(sizeof(guint32));
         mailpos[0] = 0;
         num = 1;
      }
      else
      {
         close(fd);
         fprintf(stderr, "%s is corrupt.\n", mbx);
         fflush(stderr);
         return 0;
      }

      p += strlen("From ");
      while ((p = strstr(p, "\nFrom ")))
      {
         /* this might be the beginning of a new mbox message... 
          * to make sure we must advance to the next line and read it */
         r = p + strlen("\nFrom ");
         for ( ; *r != '\n' && *r != '\0'; r++);

         /* if it says "Return-Path: " then it *is* the beginning */
         /* Uuuuh... no, not really.  Do a REAL header check */
         if (mbox_next_after_from(r))
         {
            mailpos = g_realloc(mailpos, (num + 1) * sizeof(guint32));
            mailpos[num] = (guint32)(p - q); /* yuk  */
            num++;
         }
         p = r; 
      }
      munmap(q, st.st_size);
   }

   close(fd);

   /* lets mark the end with a 0 */
   mailpos = g_realloc(mailpos, (num + 1) * sizeof(guint32));
   mailpos[num] = 0;

   fprintf (stderr, "Total messages for %s = %d\n", mbx, num);
   fflush(stderr);

   return num;
}

gchar *mbox_get_mesg(guint mesg_num, gchar *mbx)
{
   gchar *data = NULL;
   gchar *p, *q, *r, *s;
   struct stat st;
   gint fd;
   guint32 n, offset, pagesize;

   fd = open(mbx, O_RDONLY);
   if (fd == -1)
   {
      perror("open()");
      return NULL;
   }

   if (stat(mbx, &st) == -1)
   {
      perror("stat()");
      close(fd);
      return NULL;
   }

   if (st.st_size)
   {
      pagesize = getpagesize();
      offset = (mailpos[mesg_num - 1] / pagesize) * pagesize;

      p = q = (gchar *)mmap(0, st.st_size, PROT_READ, MAP_PRIVATE, fd, offset);
      if (p == (gchar*)MAP_FAILED)
      {
         close(fd);
         perror("mmap()");
         return NULL;
      }

      p += mailpos[mesg_num - 1] % pagesize;
      if (mesg_num != 1)
         p++; /* seek to just past the \n */
      n = 0;

      for (r = p; *r != '\0'; r++)
      {
         if (*r == '\n')
         {
            if (!g_strncasecmp(r, "\nFrom ", strlen("\nFrom ")))
            {
               /* skip ahead to the next line */
               for (s = r + strlen("\nFrom "); *s != '\0' && *s != '\n'; s++);
               if (mbox_next_after_from(s))
               {
                  n = (guint32) (r - p);  /* set the size of the email */
                  break;
               }
            }
         }
      }

      if (*r == '\0')
         n = (guint32) ((r-1) - p);  /* musta been the last email */

      if (n > 0)
      {
         data = g_strndup(p, n);

         /* we should just strip the message here, makes more sense */
         strip(data, '\r');
      }

      munmap(q, st.st_size);
	}

   close(fd);

   return data;
}

/* Must sort messages BEFORE calling this function */
gint mbox_del_mesgs (guint mesgs, guint *mesg_num, gchar *mbox)
{
   /* delete the specified message(s) by first copying the entire mailbox
    * to a temp file (except the message to be deleted) and then copying
    * it back. */
   gchar *tmbx = get_mailbox("_TempDel_");
   gint fd;
   gchar *mesg = NULL;
   gint num = 1, i = 0;

   if (*mesg_num <= 0)
      return 0;

   /* open the temp mbox */
   fd = open(tmbx, O_WRONLY | O_CREAT | O_TRUNC);
   if (fd == -1)
   {
      g_free(tmbx);
      return 0;
   }

   /* set secure permissions */
   chmod(tmbx, S_IREAD | S_IWRITE);

   while (i < mesgs)
   {
      /* get all the messages up to the one to be deleted */
      while (num < mesg_num[i])
      {
         mesg = mbox_get_mesg(num, mbox);
         write(fd, mesg, strlen(mesg));
         write(fd, "\n", 1);
         g_free(mesg);
         num++;
      }

      /* skip the next message - it's one of the ones to be deleted */
      num++;

      /* increment the counter that holds the number of messages deleted so far */
      i++;
   }

   /* get the rest... */
   while (mailpos[num-1]) /* last mailpos element should be 0 */
   {
      mesg = mbox_get_mesg(num, mbox);
      write(fd, mesg, strlen(mesg));
      write(fd, "\n", 1);
      g_free(mesg);
      num++;
   }

   close(fd);

   /* new method...maybe this'll make it faster */
   if (!mbox_rename(tmbx, mbox))
      fprintf(stderr, "mbox_rename(): %s failed to open for writing.\n", mbox);

   g_free(tmbx);

   return 1;
}

/* Must sort messages BEFORE calling this function */
gint mbox_copy_mesgs (guint mesgs, guint *mesg_num, gchar *orig_mbox, gchar *new_mbox)
{
   gchar *mesg, *mesg2;
   FILE *fp;
   guint i = 0;
   gboolean mboxfmt;

   /* find out the format of the new mailbox so we know if we have to convert formats later */
   mboxfmt = mailbox_is_mbox_format(new_mbox, FALSE);

   fp = fopen(new_mbox, "at");
   if (fp == NULL)
   {
	   return 0;
   }

   while (i < mesgs)
   {
      mesg = mbox_get_mesg(mesg_num[i], orig_mbox);
      if (mesg == NULL)
      {
         fflush(fp);
         fclose(fp);
         return 0;
      }

      if (mboxfmt)
      {
         fprintf(fp, "%s", mesg);
      }
      else
      {
         /* convert the mbox formatted message to spruce format and then dump it into the new mailbox */
         mesg2 = mbox_to_spruce(mesg);
         fprintf(fp, "%s", mesg2);
         g_free(mesg2);
      }

      g_free(mesg);
      i++;
   }

   fflush(fp);
   fclose(fp);

   return 1;
}

/* Must sort messages BEFORE calling this function */
gint mbox_move_mesgs (guint mesgs, guint *mesg_num, gchar *orig_mbox, gchar *new_mbox)
{
   if (!mbox_copy_mesgs(mesgs, mesg_num, orig_mbox, new_mbox))
      return 0;
   if (!mbox_del_mesgs(mesgs, mesg_num, orig_mbox))
      return 0;

   return 1;
}
