/*  Spruce
 *  Copyright (C) 1999-2000 Jeffrey Stedfast
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#	include "config.h"
#endif

#include "msgid.h"

GHashTable *msgids = NULL;

gboolean hash_free(gpointer key, gpointer value, gpointer fp)
{
   if (key)
   {
      fprintf((FILE *) fp, "%s\n", (gchar *) key);
      g_free(key);
   }
 
   return TRUE;
}

/* We have no foolproofing  */
gint open_msgids_file(void)
{
   gchar *home_dir, *msgids_file, *hgc1;
   gchar msgid[256];
   FILE *fp;

   home_dir = g_strdup(getenv("HOME"));
   if (!home_dir)
   {
       g_warning("Can't determine home dir !");
       gtk_exit(1);
   }

   if (msgids)
      g_hash_table_destroy(msgids);

   msgids = g_hash_table_new(g_str_hash, g_str_equal);
   msgids_file = g_malloc0(strlen(home_dir) + strlen(MSGIDS_FILE) + 1);
   strcpy(msgids_file, home_dir);
   strcat(msgids_file, MSGIDS_FILE);

   fp = fopen(msgids_file, "rt");

   if (fp != NULL)
   {
      while (!feof(fp))
      {
         memset(msgid, 0, sizeof(msgid));
         fgets(msgid, sizeof(msgid)-1, fp);
         trim_whtspc(msgid);
         if (*msgid != '<')
           continue;
         hgc1 = strchr(msgid, '>');
         if (hgc1)
            *(++hgc1) = '\0';
         g_hash_table_insert(msgids, g_strdup(msgid), GUINT_TO_POINTER(1));
      }
      fclose(fp);
   }

   g_free(home_dir);
   g_free(msgids_file);

   return 0;
}

gboolean find_msgid(gchar *msgid)
{
   return g_hash_table_lookup(msgids, msgid) != NULL;
}

void add_msgid(gchar *msgid)
{
   g_hash_table_insert(msgids, g_strdup(msgid), GUINT_TO_POINTER(1));
} 

void close_msgids_file(void)
{
   FILE *fp;
   gchar *home_dir, *msgids_file;
   
   home_dir = g_strdup(getenv("HOME"));
   if (!home_dir)
	{
	    g_warning("Can't determine home dir!");
	    gtk_exit(1);
	}    

   msgids_file = g_malloc0(strlen(home_dir) + strlen(MSGIDS_FILE) + 1);
   strcpy(msgids_file, home_dir);
   strcat(msgids_file, MSGIDS_FILE);

   fp = fopen(msgids_file, "wt");
   if (fp != NULL)
	{
      g_hash_table_foreach_remove(msgids, hash_free, fp);
      fclose(fp);
	}

   g_hash_table_destroy(msgids);
   g_free(home_dir);
   g_free(msgids_file);
}
