/*  Spruce
 *  Copyright (C) 1999-2000 Jeffrey Stedfast
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#ifdef ENABLE_GPG

#include "pgp.h"
#include "main.h"

extern GtkWidget *frmComposer;
extern GtkWidget *dlgPassphrase;
extern gboolean PassphraseOpen;
extern gchar *pgp_passphrase;

static gboolean called_from_decrypt;

GtkWidget *create_dlgPassphrase (gboolean cfd)
{
  GtkWidget *dialog_vbox1;
  GtkWidget *hbox1;
  GtkWidget *lblPassphrase;
  GtkWidget *txtPassphrase;
  GtkWidget *dialog_action_area1;
  GtkWidget *cmdPgpOK;
 
  called_from_decrypt = cfd;

  dlgPassphrase = gtk_dialog_new ();
  gtk_widget_set_name (dlgPassphrase, "dlgPassphrase");
  gtk_object_set_data (GTK_OBJECT (dlgPassphrase), "dlgPassphrase", dlgPassphrase);
  gtk_signal_connect (GTK_OBJECT (dlgPassphrase), "destroy",
                      GTK_SIGNAL_FUNC (on_dlgPassphrase_destroy), NULL);
  gtk_window_set_title (GTK_WINDOW (dlgPassphrase), _("Enter PGP passphrase"));
  gtk_window_set_policy (GTK_WINDOW (dlgPassphrase), TRUE, TRUE, FALSE);
  gtk_window_set_modal (GTK_WINDOW (dlgPassphrase), TRUE);

  dialog_vbox1 = GTK_DIALOG (dlgPassphrase)->vbox;
  gtk_widget_set_name (dialog_vbox1, "dialog_vbox1");
  gtk_object_set_data (GTK_OBJECT (dlgPassphrase), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  hbox1 = gtk_hbox_new (FALSE, 6);
  gtk_widget_set_name (hbox1, "hbox1");
  gtk_object_set_data (GTK_OBJECT (dlgPassphrase), "hbox1", hbox1);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox1, TRUE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (hbox1), 10);

  lblPassphrase = gtk_label_new (_("Passphrase:"));
  gtk_widget_set_name (lblPassphrase, "lblPassphrase");
  gtk_object_set_data (GTK_OBJECT (dlgPassphrase), "lblPassphrase", lblPassphrase);
  gtk_widget_show (lblPassphrase);
  gtk_box_pack_start (GTK_BOX (hbox1), lblPassphrase, TRUE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (lblPassphrase), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (lblPassphrase), 1.93715e-07, 0.5);

  txtPassphrase = gtk_entry_new ();
  gtk_widget_set_name (txtPassphrase, "txtPassphrase");
  gtk_object_set_data (GTK_OBJECT (dlgPassphrase), "txtPassphrase", txtPassphrase);
  gtk_widget_show (txtPassphrase);
  gtk_box_pack_start (GTK_BOX (hbox1), txtPassphrase, TRUE, TRUE, 0);
  gtk_entry_set_visibility (GTK_ENTRY (txtPassphrase), FALSE);

  dialog_action_area1 = GTK_DIALOG (dlgPassphrase)->action_area;
  gtk_widget_set_name (dialog_action_area1, "dialog_action_area1");
  gtk_object_set_data (GTK_OBJECT (dlgPassphrase), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  cmdPgpOK = gtk_button_new_with_label (_("OK"));
  gtk_widget_set_name (cmdPgpOK, "cmdPgpOK");
  gtk_object_set_data (GTK_OBJECT (dlgPassphrase), "cmdPgpOK", cmdPgpOK);
  gtk_widget_show (cmdPgpOK);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), cmdPgpOK, TRUE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (cmdPgpOK), "clicked",
                      GTK_SIGNAL_FUNC (on_cmdPgpOK_clicked),
                      GTK_OBJECT(dlgPassphrase));

  return dlgPassphrase;
}

void on_dlgPassphrase_destroy (GtkWidget *widget, gpointer user_data)
{
   PassphraseOpen = FALSE;
   gtk_widget_destroy(widget);
}

void on_cmdPgpOK_clicked (GtkWidget *widget, gpointer user_data)
{
   GtkWidget *txtPassphrase = get_widget(dlgPassphrase, "txtPassphrase");

   g_free(pgp_passphrase);
   pgp_passphrase = NULL;

   pgp_passphrase = g_strdup(gtk_entry_get_text(GTK_ENTRY(txtPassphrase)));

   /* destroy this window */
   gtk_widget_destroy(widget);
   /* re-emit the clicked signal to invoke the pgp encrypt function in composer.c */
   if (called_from_decrypt)
   {
	   called_from_decrypt = FALSE;
	   refresh_txtMessage();
   }
}

gint pgp_detect(gchar *text)
{
   if (strstrcase(text, "-----BEGIN PGP MESSAGE-----"))
      return 1;
   return 0;
}

gchar *pgp_encrypt(gchar *text, gchar *recipients, gint use_gnupg, gint pgp_sign)
{
   PggEncrypt encrypt   = NULL;
   PggStdio stdio   = NULL;
   PggOutopts opts  = NULL;
   PggInput input   = NULL;
   PggOutput output = NULL;
   PggBuffer result = NULL;
   PggErrenv errenv;
   gchar *encrypted_text, *buffer;
   gboolean pgp_error = FALSE;
   gint i, max;

   if (!use_gnupg)
   {
      fprintf(stderr, "Unsupported version of PGP...aborting\n");
      return NULL;
   }

   /* reset the PggErrenv object */
   pgg_errenv_reset(errenv);
   /* set the stdio for Pgg */
   stdio = pgg_stdio_new(errenv);
   /* We create a new PggInput object to determine the source of the
    * data we wish to encrypt. Then we set the raw C string 
    * with our message as source. After that, we add the PggInput object
    * to the PggStdio object, which will be used by PggEncrypt later. */
   input = pgg_input_new(errenv);
   pgg_input_set_rawdata(input, text, strlen(text), errenv);
   pgg_stdio_set_input(stdio, input, errenv);
   /* The same as for input is needed for the resulting encrypted 
    * output. However, because we don't know how large the encrypted
    * message will be, we create a PggBuffer thats purpose is to
    * manage dynamically growing memory. Additionally we need
    * a PggOutput object set with the PggBuffer as destination
    * to write to. The PggOutput object is added to the PggStdio. */
   result = pgg_buffer_new(errenv);
   output = pgg_output_new(errenv);
   pgg_output_set_buffer(output, result, errenv);
   pgg_stdio_set_output(stdio, output, errenv);
   /* Because we would like to produce ASCII armored output and 
    * would like to submit our own comment in the encrypted message,
    * we create a PggOutopts object and request the desired options
    * and finally add this PggOutopts object to the PggStdio
    * object. */
   opts = pgg_outopts_new(errenv);
   pgg_outopts_set_armor(opts, 1, errenv);         /* ascii armored output */
   pgg_outopts_set_comment(opts, "", errenv);
   pgg_outopts_set_version(opts, 0, errenv);	   /* don't create a version string */
   pgg_stdio_set_outopts(stdio, opts, errenv);
   /* A PggEncrypt is the actual workhorse used for encrypting 
    * a message conventionally. We set the recipients used for encryption
    * and the PggStdio object to specify the source and destination 
    * of all data. */
   encrypt = pgg_encrypt_new(errenv);
   pgg_encrypt_set_stdio(encrypt, stdio, errenv);
   /* set recipients */
   max = get_num_fields(recipients, ' ');
   for (i = 0; i < max; i++)
   {
      buffer = get_field(recipients, i+1, ' ');
      if (buffer)
      {
         pgg_encrypt_add_recipient(encrypt, buffer, errenv);
         g_free(buffer);
         buffer = NULL;
      }		
   }
   /* This call actually does the complete work of calling GnuPG and 
    * parsing GnuPG messages and outputs. When this call is finished,
    * the encrypted message is stored in the PggBuffer named `result'. */
   pgg_encrypt_execute(encrypt, errenv);
   /* Because we would like to have a C string as result of the 
    * enrypted data, we need to append a simple 0 character at
    * the end of the encrypted data. */
   pgg_buffer_append(result, "", 1, errenv);
   /* Now we simple print the resulting message. We don't submit the
    * error environment because it is very unlikly that this call will
    * fail if all previous calls succeded. However, in generall it's
    * always a good idea to catch all possible errors.
    */
   encrypted_text = g_strdup(pgg_buffer_get_data(result, NULL));

   pgp_error = pgg_errenv_is_set(errenv);

   /* if we had encountered an error before... */
   if (pgp_error)
   {
      fprintf(stderr, "failed to encrypt.\n");
      g_free(encrypted_text);
      return NULL;
   }

   /* cleanup */
   pgg_buffer_release(result, NULL);
   pgg_output_release(output, NULL);
   pgg_outopts_release(opts, NULL);
   pgg_input_release(input, NULL);
   pgg_stdio_release(stdio, NULL);
   pgg_encrypt_release(encrypt, NULL);

   return encrypted_text;
}

gint get_passphrase(PggPasscache passcache, PggPasscache_CbCmd cmd, const char * arg)
{
	GtkWidget *passdlg;

	switch (cmd) {
		case PGG_PASSCACHE_CMD_INIT:
			break;
		case PGG_PASSCACHE_CMD_RELEASE:
			break;
		case PGG_PASSCACHE_CMD_PASSPHRASE:
			if (pgp_passphrase == (gchar *)NULL) {
				passdlg = create_dlgPassphrase(TRUE);
				gtk_widget_show(passdlg);
				return -1;
			} else {
				pgg_passcache_set_passphrase(passcache, arg, pgp_passphrase, NULL);
			}
			break;
		default:
			return -1;
	}
	return 0;
}
			

gchar *pgp_decrypt(gchar *text, gchar *passphrase, gint use_gnupg)
{
   PggDecrypt decrypt   = NULL;
   PggStdio stdio   = NULL;
   PggOutopts opts  = NULL;
   PggInput input   = NULL;
   PggOutput output = NULL;
   PggBuffer result = NULL;
   PggErrenv errenv;
   PggPasscache passcache = NULL;
   gboolean pgp_error = FALSE;
   gchar *decrypted_text = NULL;

   if (!use_gnupg)
   {
      fprintf(stderr, "Unsupported version of PGP...aborting\n");
      return NULL;
   }

   /* reset the PggErrenv object */
   pgg_errenv_reset(errenv);
   /* set the stdio for Pgg */
   stdio = pgg_stdio_new(errenv);
   /* We create a new PggInput object to determine the source of the
    * data we wish to encrypt. Then we set the raw C string 
    * with our message as source. After that, we add the PggInput object
    * to the PggStdio object, which will be used by PggSymcrypt later. */
   input = pgg_input_new(errenv);
   pgg_input_set_rawdata(input, text, strlen(text), errenv);
   pgg_stdio_set_input(stdio, input, errenv);
   /* The same as for input is needed for the resulting encrypted 
    * output. However, because we don't know how large the encrypted
    * message will be, we create a PggBuffer thats purpose is to
    * manage dynamically growing memory. Additionally we need
    * a PggOutput object set with the PggBuffer as destination
    * to write to. The PggOutput object is added to the PggStdio. */
   result = pgg_buffer_new(errenv);
   output = pgg_output_new(errenv);
   pgg_output_set_buffer(output, result, errenv);
   pgg_stdio_set_output(stdio, output, errenv);
   /* Because we would like to produce ASCII armored output and 
    * would like to submit our own comment in the encrypted message,
    * we create a PggOutopts object and request the desired options
    * and finally add this PggOutopts object to the PggStdio
    * object. */
   opts = pgg_outopts_new(errenv);
   pgg_outopts_set_armor(opts, 1, errenv);         /* ascii armored output */
   pgg_outopts_set_comment(opts, "", errenv);      /* user defined comment string */
   pgg_outopts_set_version(opts, 0, errenv);       /* don't create a version string */
   pgg_stdio_set_outopts(stdio, opts, errenv);
   /* A PggSymcrypt is the actual workhorse used for encrypting 
    * a message conventionally. We set the passphrase used for encryption
    * and the PggStdio object to specify the source and destination 
    * of all data. */
   decrypt = pgg_decrypt_new(errenv);
   pgg_decrypt_set_stdio(decrypt, stdio, errenv);
   /* PGG uses a passcache to handle key/passphrase data.
	* This is where we setup the passcache, a user is prompted
	* for the passphrase if it is not found or invalid. */
   passcache = pgg_passcache_new(errenv);
   pgg_passcache_set_callback(passcache, get_passphrase, errenv);
   pgg_passcache_set_policy(passcache, PGG_PASSCACHE_POLICY_REFRESH, errenv);
   pgg_passcache_set_timeout(passcache, 600, errenv); /* Not sure what time limit
														 this should realy be */
   pgg_decrypt_set_passcache(decrypt, passcache, errenv);

	
   /* This call actually does the complete work of calling GnuPG and 
    * parsing GnuPG messages and outputs. When this call is finished,
    * the encrypted message is stored in the PggBuffer named `result'. */
   pgg_decrypt_execute(decrypt, errenv);
   /* Because we would like to have a C string as result of the 
    * enrypted data, we need to append a simple 0 character at
    * the end of the encrypted data. */
   pgg_buffer_append(result, "", 1, errenv);
   /* Now we simple print the resulting message. We don't submit the
    * error environment because it is very unlikly that this call will
    * fail if all previous calls succeded. However, in generall it's
    * always a good idea to catch all possible errors.
    */
   decrypted_text = g_strdup(pgg_buffer_get_data(result, NULL));

   pgp_error = pgg_errenv_is_set(errenv);

   /* if we had encountered an error before... */
   if (pgp_error)
   {
      fprintf(stderr, "failed to decrypt.\n");
      g_free(decrypted_text);
      return NULL;
   }

   /* cleanup */
   pgg_buffer_release(result, NULL);
   pgg_output_release(output, NULL);
   pgg_outopts_release(opts, NULL);
   pgg_input_release(input, NULL);
   pgg_stdio_release(stdio, NULL);
   pgg_decrypt_release(decrypt, NULL);


   return decrypted_text;
}

#endif /* ENABLE_GPG */
