#!/usr/bin/perl -w
#
#  makepo.pl
#  Copyright (C) 1999 Jeffrey Stedfast
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#  Description: This perl script makes .po files from .pot files
#
#  Requires module: WWW::Babelfish
#
#  Usage: makepo.pl <potfile> <language>
#
#  eg.: makepo.pl spruce.pot French > fr.po
#
#  valid values for language are: French, Portuguese, German, Italian, and Spanish
#  Changes:
#   11/18/1999 - Rusty Conover (rusty@zootweb.com) 
#                *Added support for dynamic languages so that we get the 
#                 list from the babelfist object and allow only valid selections
#                 also added some dummy usage code.


use WWW::Babelfish;
use strict;


my $lang_check_obj = new WWW::Babelfish('agent' => 'Mozilla/8.0');
my $lang_temp;
my $lang_hash;

foreach $lang_temp ($lang_check_obj->languages())
{
   $lang_hash->{$lang_temp} = 1;
}



if(scalar(@ARGV) < 2)
{
   print "Please specify a pot file and a language!\n";
   print "Languages Available:\n";
   foreach $lang_temp ($lang_check_obj->languages())
   {
      print "$lang_temp\n";
   }
   exit(0);
}

my ($potfile,$lang) = @ARGV;


#do the language check
if(!$lang_hash->{$lang})
{
   print "The languages that is specified is not available for\ntranslation please select an available language below:\n";
   foreach $lang_temp ($lang_check_obj->languages())
   {
      print "$lang_temp\n";
   }
   exit(0);
}

if (! -r $potfile)
{
   die "Can't read $potfile\n";
}
if (! -f $potfile)
{
   die "$potfile is not a plain file\n";
}
open(INPUT, "<$potfile") or die "Can't input $potfile $!";

my $input;

while ($input=<INPUT>)
{
   print STDOUT $input;
   if ( $input =~ /^msgid/ )
   {
      chop($input);                        # chop off \n
      my ($msgid,$text) = split(/ /, $input, 2);

      $input=<INPUT>;                      # get the next line -> we'll write a replacement

      if ($text eq '""')
      {
         print STDOUT "msgstr \"\"\n";
      }
      else
      {
         chop($text);                      # chop off trailing "
         substr($text, 0, 1) = "";         # chop off leading "

         # now we need to query babelfish
         my $obj = new WWW::Babelfish( 'agent' => 'Mozilla/8.0' );

	   
         die( "Babelfish server unavailable\n" ) unless defined($obj);

         my $translated_text = $obj->translate('source' => 'English', 'destination' => $lang, 'text' => $text);
         #die("Could not translate \"" . $text . "\": " . $obj->error) unless defined($translated_text);
         if ( defined($translated_text) )
         {
            print STDOUT "msgstr \"",$translated_text,"\"\n";
         }
         else
         {
            print STDERR "Could not translate \"",$text,"\" so leaving alone.\n";
            print STDOUT "msgstr \"",$text,"\"\n";
         }
      }
   }
}

close INPUT;
exit;
