/*  Spruce
 *  Copyright (C) 1999-2000 Jeffrey Stedfast
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#	include "config.h"
#endif

#include "rc.h"

/* external variables */
extern gchar *spruceconf;                       /* spruce config file */
extern GList *filters;
extern gint NumMailboxes;
/* Identity */
extern gchar *SenderAddr;
extern gchar *ReplyTo;
extern gint use_rand_sig;
extern gint auto_attach_sig;
extern gchar *signaturefile;
/* Accounts */
extern GList *PopAccounts;
extern GList *ImapAccounts;
extern Server SmtpServer;
extern gint external_fetchmail;
/* Appearance */
extern gchar *mesg_body_font;
extern gchar *unread_mesg_header_font;
extern gchar *read_mesg_header_font;
/* Misc */
extern gint save_outgoing;
extern gint always_prepend;
extern gint word_wrap_len;
extern gint empty_on_exit;
extern gint save_sizes;
extern gint cm_delay;
extern gint hdr_trim_level;
/* PGP */
extern gchar *pgp_identity;
extern gint use_gnupg;
extern gint pgp_sign;
extern gint encrypt_to_self;
extern gint store_passphrase;
extern gchar *pgp_passphrase;

extern gint sort_column;            /* column to sort */
extern GtkSortType sort_type;

extern GtkWidget *frmMain;

static gchar eoln = '\n';
static gchar comment = '#';


gchar *get_name_from_user(gchar *user)
{
   FILE *fp;
   gchar buffer[512];

   fp = fopen("/etc/passwd", "rt");
   if (fp == NULL)
   {
      return NULL;
   }

   while (!feof(fp))
   {
      memset(buffer, 0, sizeof(buffer));
      fgets(buffer, sizeof(buffer)-1, fp);
      if (!strncmp(buffer, user, strlen(user)))
      {
         fclose(fp);
         return get_field(buffer, 5, ':');
      }
   }

   fclose(fp);
   return NULL;
}

gint get_email_from_text (gchar *addr, gchar *text, gint maxlen)
{
   /* get the actual email address from the string passed and place it in *addr
    * we can assume the address will be in one of the following forms:
    * 1) The Name <person@host.com>
    * 2) <person@host.com>
    * 3) person@host.com
    * 4) person@host.com (The Name)
   */
   
   gchar *addr_strt;         /* points to start of addr */
   gchar *addr_end;          /* points to end of addr */
   gchar *ptr1, *ptr2;
   
   
   /* check the incoming args */
   if (!addr || !text || *text=='\0')
      return ERROR;

   /* scan the string for an open brace */
   for (addr_strt = text; *addr_strt; addr_strt++) 
      if (*addr_strt == '<')
         break;

   if (*addr_strt)  /* we found an open brace */
   {
      /* let's look for it's counterpart */
      for (addr_end = addr_strt; *addr_end; addr_end++)
         if(*addr_end == '>') 
            break;

      /* if we didn't find it, or braces are empty... */
      if (!(*addr_end) || (addr_strt == addr_end - 1))
         return ERROR;
			
      /* addr_strt points to '<' and addr_end points to '>'.
       * Now let's adjust 'em slightly to point to the beginning
       * and ending of the email addy
		 */
      addr_strt++;
      addr_end--;
   }
   else    /* no open brace...assume type 3 or 4? */
   {
      addr_strt = text;
			
      /* find the end of the email addr/string */
      for (addr_end = addr_strt; *addr_end || *addr_end == ' '; addr_end++);
 
      addr_end--;       /* points to NULL, move it back one char */
   }

   /* now addr_strt & addr_end point to the beginning & ending of the email addy */

   /* copy the string into addr */
   for (ptr1 = addr_strt, ptr2 = addr; ptr1 <= addr_end && (ptr2 - addr) < maxlen; ptr1++, ptr2++)
      *ptr2 = *ptr1;
   *ptr2 = '\0';

   for (ptr1 = addr_strt; ptr1 <= addr_end; ptr1++)    /* look for an '@' sign */
      if (*ptr1 == '@')
         break;

   if (*ptr1 != '@' && (ptr2 - addr) < (maxlen - 1))
   {
      /* here we found out the name doesn't have an '@' part
       * let's figure out what machine we're on & stick it on the end
       * woops.. forgot the '@' sign, gotta stick that in the reply
       * ptr2 should still point to the next place to copy to in addr
       */
      
      *ptr2++ = '@';
      if (gethostname(ptr2, maxlen - (ptr2 - addr) - 1))
         return ERROR;

      /* fast forward to the end of the hostname */
      for ( ; *ptr2; ptr2++);

      *ptr2++ = '.';
      getdomainname(ptr2, maxlen - (ptr2 - addr) - 1);
      if (*ptr2 == '(')
         *(ptr2-1) = '\0';
   }
   else               /* there's an @ sign already in the email addy */
      *ptr2 = '\0';   /* add a null to the end of the string */

   return SUCCESS;
}

gint rc_save_defaults()
{
   GtkWidget *treelocalBoxes = get_widget(frmMain, "treelocalBoxes");
   GtkWidget *treespoolBoxes = get_widget(frmMain, "treespoolBoxes");
   GtkWidget *clistMail  = get_widget(frmMain, "clistMail");
   GtkWidget *pan1 = get_widget(frmMain, "pan1");
   GtkWidget *pan2 = get_widget(frmMain, "pan2");
   GtkWidget *pan3 = get_widget(frmMain, "pan3");
   FILE *fp;
   gint i;
   gchar *mailbox, *user, *name, email[1024], *ptr;
   GList *tmp;
   struct filter *flt;
   PopAccount_t *pop = NULL;
   ImapAccount_t *imap = NULL;

   if (!spruceconf)
   {
      fprintf(stderr, "Spruce: configuration file unknown.\n");
      return 0;
   }

   fp = fopen(spruceconf, "wt");
   if (fp == NULL)
   {
      fprintf(stderr, "Spruce: %s failed to open for writing.\n", spruceconf);
      fprintf(stderr, "Please check with your local Administrator.\n");
      return 0;
   }
   chmod(spruceconf, 0600);

   fprintf(fp, "%c This is the Spruce initialization file.\n", comment);
   fprintf(fp, "%c Lines that start with a '%c' are comments.\n", comment, comment);
   fprintf(fp, "%c Blank lines are ignored.\n\n", comment);

   fprintf(fp, "%c Set the SMTP server information\n", comment);
   if (SmtpServer.hostname != NULL)
      fprintf(fp, "smtp-server %s\n", SmtpServer.hostname);
   else
      fprintf(fp, "%c smtp-server\n", comment);
   fprintf(fp, "smtp-port %d\n\n", SmtpServer.port);

   fprintf(fp, "%c Specify whether or not Spruce should save outgoing emails.\n", comment);
   fprintf(fp, "%c 0 = no, 1 = yes\n", comment);
   fprintf(fp, "save-outgoing %d\n\n", save_outgoing);

   fprintf(fp, "%c Set the user's name and address...\n", comment);
   if (SenderAddr != NULL)
      fprintf(fp, "email-addr %s\n\n", SenderAddr);
   else
   {
      user = g_strdup(getenv("USER"));
      name = get_name_from_user(user);
      if (!name || !*name)
         name = g_strdup(user);
      for (ptr = name; *ptr && *ptr != ','; ptr++);
      *ptr = '\0';
      memset(email, 0, sizeof(email));
      get_email_from_text(email, user, 1023);
      fprintf(fp, "email-addr %s <%s>\n\n", name, email);
      g_free(user);
      g_free(name);
   }

   fprintf(fp, "%c Set the user's Reply-To address (assuming it differs from their email-addr)\n", comment);
   if (ReplyTo != NULL)
      fprintf(fp, "reply-to-addr %s\n\n", ReplyTo);
   else
      fprintf(fp, "%c reply-to-addr joe@sixpack.org\n\n", comment); 

   fprintf(fp, "%c Specify if you want Spruce to use a random signature.\n", comment);
   fprintf(fp, "%c 0 = off, 1 = on\n", comment);
   fprintf(fp, "use-rand-sig %d\n\n", use_rand_sig);

   fprintf(fp, "%c Auto attach signature\n", comment);
   fprintf(fp, "%c 0 = no, 1 = yes\n", comment);
   fprintf(fp, "auto-attach-sig %d\n\n", auto_attach_sig);

   fprintf(fp, "%c Specify the location of your personal signature file.\n", comment);
   if (signaturefile != NULL)
      fprintf(fp, "sig-file %s\n\n", signaturefile);
   else
      fprintf(fp, "%c sig-file\n\n", comment);

   fprintf(fp, "%c Message font (only affects the GUI client)\n", comment);
   if (mesg_body_font != NULL)
      fprintf(fp, "mesg-body-font \"%s\"\n\n", mesg_body_font);
   else
      fprintf(fp, "%c mesg-body-font\n\n", comment);

   fprintf(fp, "%c Unread message header font \n", comment);
   if (unread_mesg_header_font != NULL)
      fprintf(fp, "unread-mesg-header-font \"%s\"\n\n", unread_mesg_header_font);
   else
      fprintf(fp, "%c unread-mesg-header-font\n\n", comment);

   fprintf(fp, "%c Read message header font \n", comment);
   if (read_mesg_header_font != NULL)
      fprintf(fp, "read-mesg-header-font \"%s\"\n\n", read_mesg_header_font);
   else
      fprintf(fp, "%c read-mesg-header-font\n\n", comment);

   fprintf(fp, "%c List your mailboxes here.\n", comment);
   if (NumMailboxes && treelocalBoxes != NULL)
   {
      GList *items;

      items = GTK_TREE(treelocalBoxes)->children;
      while (items != NULL)
      {
      	gtk_label_get (GTK_LABEL (GTK_BIN (GTK_WIDGET(items->data))->child), &mailbox);
         fprintf (fp, "mailbox %s\n", mailbox);
         items = items->next;
      }
   }
   else
   {
      fprintf(fp, "mailbox Inbox\n");
      fprintf(fp, "mailbox Drafts\n");
      fprintf(fp, "mailbox Outbox\n");
      fprintf(fp, "mailbox Sent items\n");
      fprintf(fp, "mailbox Deleted items\n");
   }
   fprintf(fp, "\n");

   fprintf(fp, "%c List your spool mbox's here (aka /var/spool/mail).\n", comment);
   if (treespoolBoxes != NULL)
   {
      GList *items;

      items = GTK_TREE(treespoolBoxes)->children;

      if (items != NULL)
      {
         while (items != NULL)
         {
            gtk_label_get (GTK_LABEL (GTK_BIN (GTK_WIDGET(items->data))->child), &mailbox);
            fprintf (fp, "mbox-spool %s\n", mailbox);
            items = items->next;
         }
      }
      else
      {
         fprintf(fp, "mbox-spool %s\n", getenv("MAIL"));
      }
   }
   else
   {
      fprintf(fp, "mbox-spool %s\n", getenv("MAIL"));
   }
   fprintf(fp, "\n");

   fprintf (fp, "# Use external fetchmail program\n");
   fprintf (fp, "# 0 = no, 1 = yes\n");
   fprintf (fp, "external-fetchmail %d\n\n", external_fetchmail);

   fprintf(fp, "%c List your pop3 servers here in the following format:\n", comment);
   fprintf(fp, "%c pop3-server	hostname:port:mailbox:login:passwd:del:authtype\n", comment);
   fprintf(fp, "%c\n", comment);
   fprintf(fp, "%c Note: \"mailbox\" must be one of the listed mailboxes above.\n", comment);
   for (i = 0; i < g_list_length(PopAccounts); i++)
   {
      pop = g_list_nth_data(PopAccounts, i);
      fprintf(fp, "pop3-server %s:%d:%s:%s:%s:%d:%d\n", pop->server.hostname,
              pop->server.port, pop->mailbox, pop->login, pop->passwd,
              pop->delemail, pop->authtype);
   }
   fprintf(fp, "\n");

   fprintf(fp, "%c List you imap servers here in the following format:\n", comment);
   fprintf(fp, "%c imap-server  hostname:port:login:passwd:authtype:namespace\n", comment);
   fprintf(fp, "%c\n", comment);
   i = 0;
   for (i = 0; i < g_list_length(ImapAccounts); i++)
   {
      imap = g_list_nth_data(ImapAccounts, i);
      fprintf(fp, "imap-server %s:%d:%s:%s:%d:%s\n", imap->server.hostname,
   			  imap->server.port, imap->login, imap->passwd, imap->authtype,
              imap->namespace);
   }
   fprintf(fp, "\n");

   fprintf(fp, "%c Put filters here in the following format:\n", comment);
   fprintf(fp, "%c filter filtername~type~matchtext~in~action~to\n", comment);
   fprintf(fp, "%c\n", comment);
   fprintf(fp, "%c Note: We use such a strange delimiter because otherwise\n", comment);
   fprintf(fp, "%c       we cant use ':' in the match text\n", comment);

   tmp = filters;
   while (tmp != NULL)
   {
      flt = tmp->data;
      fprintf(fp, "filter %s~%d~%s~%s~%d~%s\n", flt->name, flt->type, flt->match, flt->in, flt->action, flt->to);
      tmp = tmp->next;
   }
   fprintf(fp, "\n");

   fprintf(fp, "%c Periodic mail check.\n", comment);
   fprintf(fp, "%c example: cm-delay XX\n", comment);
   fprintf(fp, "%c Spruce will check mail every XX minutes.\n", comment);
   fprintf(fp, "%c Note: 0 means to never auto-check mail.\n", comment);
   fprintf(fp, "cm-delay %d\n\n", cm_delay);

   fprintf(fp, "%c Amount of header to show\n", comment);
   fprintf(fp, "%c 0 = none, 1 = partial, 2 = full\n", comment);
   fprintf(fp, "hdr-trim-level %d\n\n", hdr_trim_level);

   fprintf(fp, "%c Always prepend with a Re: or a Fw: when replying/forwarding?\n", comment);
   fprintf(fp, "%c 0 = no, 1 = yes\n", comment);
   fprintf(fp, "always-prepend %d\n\n", always_prepend);

   fprintf(fp, "%c Specify if Spruce should empty the \"Deleted items\" folder on exit\n", comment);
   fprintf(fp, "%c 0 = no, 1 = yes\n", comment);
   fprintf(fp, "empty-on-exit %d\n\n", empty_on_exit);

   fprintf(fp, "%c Specify the word-wrap length\n", comment);
   fprintf(fp, "word-wrap %d\n\n", word_wrap_len);

   fprintf(fp, "%c Are we going to use these sizes?\n", comment);
   fprintf(fp, "save-sizes %d\n", save_sizes);
   fprintf(fp, "%c Spruce's window size and column sizes\n", comment);
   fprintf(fp, "spruce-sizes %d:%d", GTK_WIDGET(frmMain)->allocation.width, 
			  GTK_WIDGET(frmMain)->allocation.height);
   i = 0;
   while (i < GTK_CLIST(clistMail)->columns)
   {
      fprintf(fp, ":%d", GTK_CLIST(clistMail)->column[i].width);
      i++;
   }
   /* Pane sizes below */
   fprintf(fp, ":%i:%i:%i\n\n", GTK_PANED(pan1)->child1_size, 
           GTK_PANED(pan2)->child1_size, GTK_PANED(pan3)->child1_size);

   fprintf(fp, "# How to sort the columns (internal)\n");
   fprintf(fp, "column-sort %i:%i\n\n", sort_column, sort_type);

   fprintf(fp, "%c PGP Encryption Information:\n%c\n", comment, comment);
   fprintf(fp, "%c PGP Identity\n", comment);
   if (pgp_identity)
      fprintf(fp, "pgp-identity %s\n\n", pgp_identity);
   fprintf(fp, "%c Do we want to use gpg or pgp50i? (1 = gnupg, 0 = pgp50i)\n", comment);
   fprintf(fp, "use-gnupg %d\n\n", use_gnupg);
   fprintf(fp, "%c Sign outgoing PGP messages (0 = no, 1 = yes)\n", comment);
   fprintf(fp, "pgp-sign %d\n\n", pgp_sign);
   fprintf(fp, "%c Always encrypt to self? (0 = no, 1 = yes)\n", comment);
   fprintf(fp, "enrypt-to-self %d\n\n", encrypt_to_self);
   fprintf(fp, "%c Do you want to store the pgp passphrase? (0 = no, 1 = yes)\n", comment);
   fprintf(fp, "store-passphrase %d\n\n", store_passphrase);
   if (store_passphrase && pgp_passphrase)
   {
      fprintf(fp, "%c PGP Passphrase:\n", comment);
      fprintf(fp, "pgp-passphrase \"%s\"\n\n", pgp_passphrase);
   }

   fprintf(fp, "\n%c End of Spruce initialization file.", comment);

   fflush(fp);
   fclose(fp);
	
   return 1;
}

gint rc_read_defaults()
{
   FILE *fp;
   gchar buffer[513];

   if (!spruceconf)
   {
      fprintf(stderr, "Spruce: configuration file unknown.\n");
      return 0;
   }

   fp = fopen(spruceconf, "rt");
   if (fp == NULL)
   {
      fprintf(stderr, "Spruce: %s failed to open for reading.\n", spruceconf);
      fprintf(stderr, "Please check with your local Administrator.\n");
      return 0;
   }

   while (!feof(fp))
   {
      memset(buffer, 0, sizeof(buffer));
      fgets(buffer, sizeof(buffer)-1, fp);
      trim_leadspc(buffer);
      if (buffer[0] != eoln && buffer[0] != comment)
      {
         if (!strncmp("smtp-server", buffer, strlen("smtp-server")))
         {
            strcut(buffer, 0, strlen("smtp-server"));
            trim_whtspc(buffer);
            if (*buffer == '\0')   /* if it's blank, let it be */
               continue;
            if (SmtpServer.hostname != NULL)
               g_free(SmtpServer.hostname);
            SmtpServer.hostname = g_strdup(buffer);
            continue;
         }
         if (!strncmp("smtp-port", buffer, strlen("smtp-port")))
         {
            strcut(buffer, 0, strlen("smtp-port"));
            trim_whtspc(buffer);
            SmtpServer.port = atoi(buffer);
            continue;
         }
         if (!strncmp("save-outgoing", buffer, strlen("save-outgoing")))
         {
            strcut(buffer, 0, strlen("save-outgoing"));
            trim_whtspc(buffer);
            save_outgoing = atoi(buffer);
            continue;
         }
         if (!strncmp("always-prepend", buffer, strlen("always-prepend")))
         {
            strcut(buffer, 0, strlen("always-prepend"));
            trim_whtspc(buffer);
            always_prepend = atoi(buffer);
            continue;
         }
         if (!strncmp("email-addr", buffer, strlen("email-addr")))
         {
            strcut(buffer, 0, strlen("email-addr"));
            trim_whtspc(buffer);
            if (*buffer == '\0')   /* if it's blank, let it be */
               continue;
            if (SenderAddr != NULL)
               g_free(SenderAddr);
            SenderAddr = g_strdup(buffer);
            continue;
         }
         if (!strncmp("reply-to-addr", buffer, strlen("reply-to-addr")))
         {
            strcut(buffer, 0, strlen("reply-to-addr"));
            trim_whtspc(buffer);
            if (*buffer == '\0')   /* if it's blank, let it be */
               continue;
            if (ReplyTo != NULL)
               g_free(ReplyTo);
            ReplyTo = g_strdup(buffer);
            continue;
         }
         if (!strncmp("use-rand-sig", buffer, strlen("use-rand-sig")))
         {
            strcut(buffer, 0, strlen("use-rand-sig"));
            trim_whtspc(buffer);
            use_rand_sig = atoi(buffer);
            continue;
         }
         if (!strncmp("auto-attach-sig", buffer, strlen("auto-attach-sig")))
         {
            strcut(buffer, 0, strlen("auto-attach-sig"));
            trim_whtspc(buffer);
            auto_attach_sig = atoi(buffer);
            continue;
         }
         if (!strncmp("sig-file", buffer, strlen("sig-file")))
         {
            strcut(buffer, 0, strlen("sig-file"));
            trim_whtspc(buffer);
            if (*buffer == '\0')   /* if it's blank, let it be */
               continue;
            if (signaturefile != NULL)
               g_free(signaturefile);
            signaturefile = g_strdup(buffer);
            continue;
         }
         if (!strncmp("external-fetchmail", buffer, strlen("external-fetchmail")))
         {
            strcut(buffer, 0, strlen("external-fetchmail"));
            trim_whtspc(buffer);
            external_fetchmail = atoi(buffer);
            continue;
         }
         if (!strncmp("cm-delay", buffer, strlen("cm-delay")))
         {
            strcut(buffer, 0, strlen("cm-delay"));
            trim_whtspc(buffer);
            cm_delay = atoi(buffer);
            continue;
         }
         if (!strncmp("hdr-trim-level", buffer, strlen("hdr-trim-level")))
         {
            strcut(buffer, 0, strlen("hdr-trim-level"));
            trim_whtspc(buffer);
            hdr_trim_level = atoi(buffer);
            continue;
         }
         if (!strncmp("empty-on-exit", buffer, strlen("empty-on-exit")))
         {
            strcut(buffer, 0, strlen("empty-on-exit"));
            trim_whtspc(buffer);
            empty_on_exit = atoi(buffer);
            continue;
         }
         if (!strncmp("word-wrap", buffer, strlen("word-wrap")))
         {
            strcut(buffer, 0, strlen("word-wrap"));
            trim_whtspc(buffer);
            word_wrap_len = atoi(buffer);
            continue;
         }
         if (!strncmp("pgp-identity", buffer, strlen("pgp-identity")))
         {
            strcut(buffer, 0, strlen("pgp-identity"));
            trim_whtspc(buffer);
            if (*buffer == '\0')  /* if it's blank, leave it be */
               continue;
            if (pgp_identity != NULL)
               g_free(pgp_identity);
            pgp_identity = g_strdup(buffer);
            continue;
         }
         if (!strncmp("use-gnupg", buffer, strlen("use-gnupg")))
         {
            strcut(buffer, 0, strlen("use-gnupg"));
            trim_whtspc(buffer);
            use_gnupg = atoi(buffer);
            continue;
         }
         if (!strncmp("pgp-sign", buffer, strlen("pgp-sign")))
         {
            strcut(buffer, 0, strlen("pgp-sign"));
            trim_whtspc(buffer);
            pgp_sign = atoi(buffer);
            continue;
         }
         if (!strncmp("encrypt-to-self", buffer, strlen("encrypt-to-self")))
         {
            strcut(buffer, 0, strlen("encrypt-to-self"));
            trim_whtspc(buffer);
            encrypt_to_self = atoi(buffer);
            continue;
         }
         if (!strncmp("store-passphrase", buffer, strlen("store-passphrase")))
         {
            strcut(buffer, 0, strlen("store-passphrase"));
            trim_whtspc(buffer);
            store_passphrase = atoi(buffer);
            continue;
         }
         if (!strncmp("pgp-passphrase", buffer, strlen("pgp-passphrase")))
         {
            strcut(buffer, 0, strlen("pgp-passphrase"));
            trim_whtspc(buffer);
            unquote(buffer);      /* unquote the buffer */
            if (*buffer == '\0')  /* if it's blank, leave it be */
               continue;
            if (pgp_passphrase != NULL)
               g_free(pgp_passphrase);
            pgp_passphrase = g_strdup(buffer);
            continue;
         }
         if (!strncmp("mesg-body-font", buffer, strlen("mesg-body-font")))
         {
            strcut(buffer, 0, strlen("mesg-body-font"));
            trim_whtspc(buffer);
            unquote(buffer);      /* unquote the buffer */
            if (*buffer == '\0')  /* if it's blank, leave it be */
               continue;
            if (mesg_body_font != NULL)
               g_free(mesg_body_font);
            mesg_body_font = g_strdup(buffer);
            continue;
         }

         if (!strncmp("unread-mesg-header-font", buffer, strlen("unread-mesg-header-font")))
         {
            strcut(buffer, 0, strlen("unread-mesg-header-font"));
            trim_whtspc(buffer);
            unquote(buffer);      /* unquote the buffer */
            if (*buffer == '\0')  /* if it's blank, leave it be */
               continue;
            if (unread_mesg_header_font != NULL)
               g_free(unread_mesg_header_font);
            unread_mesg_header_font = g_strdup(buffer);
            continue;
         }
	
         if (!strncmp("read-mesg-header-font", buffer, strlen("read-mesg-header-font")))
         {
            strcut(buffer, 0, strlen("read-mesg-header-font"));
            trim_whtspc(buffer);
            unquote(buffer);      /* unquote the buffer */
            if (*buffer == '\0')  /* if it's blank, leave it be */
               continue;
            if (read_mesg_header_font != NULL)
               g_free(read_mesg_header_font);
            read_mesg_header_font = g_strdup(buffer);
            continue;
         }

         if (!strncmp("save-sizes", buffer, strlen("save-sizes")))
         {
            strcut(buffer, 0, strlen("save-sizes"));
            trim_whtspc(buffer);
            save_sizes = atoi(buffer);
            continue;
         }
      }
   }

   fclose(fp);
   return 1;
}
